"""
Toolbar
=======

See: https://the-carbon-components.netlify.app/?nav=toolbar

""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import FormNode, Node, modify_svg

class Toolbar(Node):
    """Toolbar component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bx--toolbar {class}" data-toolbar {props}>
  {child}
</div>
"""
        return self.format(template, values)


class ToolbarSearch(FormNode):
    """Toolbar search component.
    """
    NODE_PROPS = ('placeholder',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_toolbar_search'] = _("Toolbar Search")
        values['txt_clear_search'] = _("Clear search input")


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].append('bx--search-input')

        placeholder = self.eval(self.kwargs.get('placeholder'), context)
        props['placeholder'] = placeholder or _("Search")


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bx--search bx--search--sm bx--toolbar-search" role="search"
    data-search data-toolbar-search>
  {tmpl_label}
  {tmpl_element}
  <button class="bx--toolbar-search__btn" aria-label="{txt_toolbar_search}">
    <svg focusable="false" preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg" fill="currentColor"
        class="bx--search-magnifier" width="16" height="16"
        viewBox="0 0 16 16" aria-hidden="true">
      <path d="M15,14.3L10.7,10c1.9-2.3,1.6-5.8-0.7-7.7S4.2,0.7,2.3,3S0.7,8.8,3,10.7c2,1.7,5,1.7,7,0l4.3,4.3L15,14.3z M2,6.5	C2,4,4,2,6.5,2S11,4,11,6.5S9,11,6.5,11S2,9,2,6.5z"></path>
    </svg>
  </button>
  <button class="bx--search-close bx--search-close--hidden"
      title="{txt_clear_search}" aria-label="{txt_clear_search}">
    <svg focusable="false" preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg" fill="currentColor" width="16"
        height="16" viewBox="0 0 32 32" aria-hidden="true">
      <path d="M24 9.4L22.6 8 16 14.6 9.4 8 8 9.4 14.6 16 8 22.6 9.4 24 16 17.4 22.6 24 24 22.6 17.4 16 24 9.4z"></path>
    </svg>
  </button>
</div>
"""
        return self.format(template, values, context)


class ToolbarItem(Node):
    """Toolbar item component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('filter',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_list'] = _("List of options")


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div data-overflow-menu class="bx--overflow-menu" tabindex="0"
    aria-label="{txt_list}">
  {slot_icon}
  <ul class="bx--overflow-menu-options">
    {child}
  </ul>
</div>
"""
        return self.format(template, values, context)


    def render_slot_icon(self, values, context):
        """Render html of the slot.
        """
        classname = ['bx--overflow-menu__icon']
        if self.eval(self.kwargs.get('filter'), context):
            classname.append('bx--toolbar-filter-icon')

        return modify_svg(values['child'], {
            'focusable': 'false',
            'preserveAspectRatio': 'xMidYMid meet',
            'fill': 'currentColor',
            'class': ' '.join(classname),
            'style': {
                'width': '%spx' % 16,
                'height': '%spx' % 16,
            },
            'aria-hidden': 'true',
        })


class ToolbarItemMultiSelect(FormNode):
    """Toolbar multi select item component.
    """
    def render_default(self, values, context):
        """Output html of the component.
        """
        template_header = """
<li class="bx--toolbar-menu__title">{label}</li>
"""
        template = """
<li class="bx--toolbar-menu__option">
  <input id="{id}" class="bx--checkbox" type="checkbox" value="{value}"
      name="{name}" {props}>
  <label for="{id}" class="bx--checkbox-label">
    {child}
  </label>
</li>
"""
        items = []

        items.append(self.format(template_header, values))

        for ii, (_, val, txt) in enumerate(self.choices()):
            options = {
                'id': '%s-%s' % (values['id'], ii),
                'index': ii,
                'value': val,
                'child': txt,
                'name': self.bound_field.name,
            }
            props = []
            if ii == 0:
                props.append('data-floating-menu-primary-focus')
            if self.bound_value and val in self.bound_value:
                props.append('checked')
            options['props'] = ' '.join(props)
            items.append(self.format(template, options))

        return '\n'.join(items)


class ToolbarItemRadioButton(FormNode):
    """Toolbar radio item component.
    """
    NODE_PROPS = ('legend',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        legend = self.eval(self.kwargs.get('legend'), context)
        values['legend'] = legend or values['label']


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<li class="bx--toolbar-menu__title">{label}</li>
<fieldset data-row-height class="bx--radio-button-group">
  <legend class="bx--visually-hidden">{legend}</legend>
  {tmpl_items}
</fieldset>
"""
        return self.format(template, values, context)


    def render_tmpl_items(self, values, context):
        """Dynamically render a part of the component's template.
        """
        template = """
<li class="bx--toolbar-menu__option">
  <input id="{id}" class="bx--radio-button" type="radio" value="{value}"
      name="{name}" {props}>
  <label for="{id}" class="bx--radio-button__label">
    <span class="bx--radio-button__appearance"></span>
    {child}
  </label>
</li>
"""
        items = []
        for ii, (_, val, txt) in enumerate(self.choices()):
            options = {
                'id': '%s-%s' % (values['id'], ii),
                'index': ii,
                'value': val,
                'child': txt,
                'name': self.bound_field.name,
            }
            props = []
            if ii == 0:
                props.append('data-floating-menu-primary-focus')
            if val == self.bound_value:
                props.append('checked')
            options['props'] = ' '.join(props)
            items.append(self.format(template, options))

        return '\n'.join(items)


class ToolbarItemButton(Node):
    """Toolbar item heading component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'button'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<li class="bx--overflow-menu-options__option">
  <{tag} class="bx--overflow-menu-options__btn {class}" {props}>
    {child}
  </{tag}>
</li>
"""
        return self.format(template, values)


class ToolbarItemDivider(Node):
    """Toolbar item divider component.
    """
    def render_default(self, values, context):
        """Output html of the component.
        """
        return '<hr class="bx--toolbar-menu__divider"/>'


components = {
    'Toolbar': Toolbar,
    'ToolbarSearch': ToolbarSearch,
    'ToolbarItem': ToolbarItem,
    'ToolbarMultiSelect': ToolbarItemMultiSelect,
    'ToolbarRadio': ToolbarItemRadioButton,
    'ToolbarButton': ToolbarItemButton,
    'ToolbarDivider': ToolbarItemDivider,
}



"""
Radio Button
============

See: https://www.carbondesignsystem.com/components/radio-button/usage/

Use radio buttons when you have a group of mutually exclusive choices and only
one selection from the group is allowed.

Overview
--------

Radio buttons are used for mutually exclusive choices, not for multiple
choices. Only one radio button can be selected at a time. When a user chooses
a new item, the previous choice is automatically deselected.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from .base import FormNode

class RadioButton(FormNode):
    """Radio Button component.
    """
    NODE_PROPS = ('exclude', 'vertical', 'left')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('radio',)
    "Prepare xxx_class and xxx_props values."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('vertical'), context):
            values['radio_class'].append('bx--radio-button-group--vertical')

        if self.eval(self.kwargs.get('left'), context):
            values['class'].append('bx--radio-button-wrapper--label-left')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<fieldset class="bx--fieldset">
  <legend class="bx--label">{label}</legend>
  <div class="bx--form-item">
    <div class="bx--radio-button-group {radio_class}">
      {tmpl_items}
    </div>
  </div>
</fieldset>
"""
        return self.format(template, values, context)


    def render_tmpl_items(self, values, context):
        """Dynamically render a part of the component's template.
        """
        template = """
<div class="bx--radio-button-wrapper {class}">
  <input id="{id}" class="bx--radio-button" type="radio" value="{value}"
      name="{name}" tabindex="0" {props}>
  <label for="{id}" class="bx--radio-button__label">
    <span class="bx--radio-button__appearance"></span>
    <span class="bx--radio-button__label-text">{child}</span>
  </label>
</div>
"""
        selected = self.bound_field.value()
        excludes = self.eval(self.kwargs.get('exclude', []), context)
        if isinstance(excludes, str):
            excludes = [x.strip() for x in excludes.split(';')]

        items = []
        for ii, (_, val, txt) in enumerate(self.choices()):
            options = {
                'id': '%s-%s' % (values['id'], ii + 1),
                'value': val,
                'child': txt,
                'name': self.bound_field.name,
                'class': values['class'],
            }
            props = []
            if val == selected:
                props.append('checked')
            if val in excludes:
                props.append('disabled')
            options['props'] = ' '.join(props)
            items.append(self.format(template, options))

        return '\n'.join(items)


components = {
    'Radio': RadioButton,
}



"""
Date Picker
===========

See: https://www.carbondesignsystem.com/components/date-picker/usage/

Date and time pickers allow users to select a single or a range of dates and
times.

Overview
--------

Pickers are used to display past, present, or future dates or times. The kind
of date (exact, approximate, memorable) you are requesting from the user will
determine which picker is best to use. Each picker’s format can be customized
depending on location or need.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from .base import FormNode, FormNodes

class DatePicker(FormNode):
    """Date Picker component.
    """
    MODES = ('default', 'basic', 'nolabel')
    "Available variants."
    NODE_PROPS = ('short', 'light')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label', 'help', 'picker')
    "Prepare xxx_class and xxx_props values."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('light'), context):
            values['picker_class'].append('bx--date-picker--light')

        if self.eval(self.kwargs.get('short'), context):
            values['picker_class'].append('bx--date-picker--short')


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].append('bx--date-picker__input')
        props['data-date-picker-input'] = ''
        if self.eval(self.kwargs.get('short'), context):
            props['pattern'] = r'\d{1,2}/\d{4,4}'
            props['placeholder'] = 'mm/yyyy'
        else:
            props['pattern'] = r'\d{1,2}/\d{1,2}/\d{4,4}'
            props['placeholder'] = 'mm/dd/yyyy'

        if self.bound_field.errors:
            props['data-invalid'] = ''


    def render_default(self, values, context):
        """Output html of the component.
        """
        if self.bound_field.errors:
            template = """
<div class="bx--form-item">
  <div data-date-picker data-date-picker-type="single"
      class="bx--date-picker bx--date-picker--single {picker_class}"
      {picker_props}>

    <div class="bx--date-picker-container">
      {tmpl_label}
      <div class="bx--date-picker-input__wrapper">
        {tmpl_element}
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            data-date-picker-icon="true" class="bx--date-picker__icon"
            width="16" height="16" viewBox="0 0 32 32" aria-hidden="true">
          <path d="M26,4h-4V2h-2v2h-8V2h-2v2H6C4.9,4,4,4.9,4,6v20c0,1.1,0.9,2,2,2h20c1.1,0,2-0.9,2-2V6C28,4.9,27.1,4,26,4z M26,26H6V12h20  V26z M26,10H6V6h4v2h2V6h8v2h2V6h4V10z"></path>
        </svg>
      </div>
      <div class="bx--form-requirement">
        {tmpl_errors}
      </div>
    </div>
  </div>
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  <div data-date-picker data-date-picker-type="single"
      class="bx--date-picker bx--date-picker--single {picker_class}"
      {picker_props}>

    <div class="bx--date-picker-container">
      {tmpl_label}
      <div class="bx--date-picker-input__wrapper">
        {tmpl_element}
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            data-date-picker-icon="true" class="bx--date-picker__icon"
            width="16" height="16" viewBox="0 0 32 32" aria-hidden="true">
          <path d="M26,4h-4V2h-2v2h-8V2h-2v2H6C4.9,4,4,4.9,4,6v20c0,1.1,0.9,2,2,2h20c1.1,0,2-0.9,2-2V6C28,4.9,27.1,4,26,4z M26,26H6V12h20  V26z M26,10H6V6h4v2h2V6h8v2h2V6h4V10z"></path>
        </svg>
      </div>
    </div>
  </div>
</div>
"""
        return self.format(template, values, context)


    def render_nolabel(self, values, context):
        """ Basic with calendar but without label.
        """
        if self.bound_field.errors:
            template = """
<div class="bx--form-item">
  <div data-date-picker data-date-picker-type="single"
      class="bx--date-picker bx--date-picker--single bx--date-picker--nolabel {picker_class}"
      {picker_props}>

    <div class="bx--date-picker-container">
      <div class="bx--date-picker-input__wrapper">
        {tmpl_element}
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            data-date-picker-icon="true" class="bx--date-picker__icon"
            width="16" height="16" viewBox="0 0 32 32" aria-hidden="true">
          <path d="M26,4h-4V2h-2v2h-8V2h-2v2H6C4.9,4,4,4.9,4,6v20c0,1.1,0.9,2,2,2h20c1.1,0,2-0.9,2-2V6C28,4.9,27.1,4,26,4z M26,26H6V12h20  V26z M26,10H6V6h4v2h2V6h8v2h2V6h4V10z"></path>
        </svg>
      </div>
      <div class="bx--form-requirement">
        {tmpl_errors}
      </div>
    </div>
  </div>
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  <div data-date-picker data-date-picker-type="single"
      class="bx--date-picker bx--date-picker--single bx--date-picker--nolabel {picker_class}"
      {picker_props}>

    <div class="bx--date-picker-container">
      <div class="bx--date-picker-input__wrapper">
        {tmpl_element}
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            data-date-picker-icon="true" class="bx--date-picker__icon"
            width="16" height="16" viewBox="0 0 32 32" aria-hidden="true">
          <path d="M26,4h-4V2h-2v2h-8V2h-2v2H6C4.9,4,4,4.9,4,6v20c0,1.1,0.9,2,2,2h20c1.1,0,2-0.9,2-2V6C28,4.9,27.1,4,26,4z M26,26H6V12h20  V26z M26,10H6V6h4v2h2V6h8v2h2V6h4V10z"></path>
        </svg>
      </div>
    </div>
  </div>
</div>
"""
        return self.format(template, values, context)


    def render_basic(self, values, context):
        """ Basic without calendar (mm/yy only).
        """
        if self.bound_field.errors:
            template = """
<div class="bx--form-item">
  <div class="bx--date-picker bx--date-picker--simple {picker_class}"
      {picker_props}>

    <div class="bx--date-picker-container">
      {tmpl_label}
      {tmpl_element}
      <div class="bx--form-requirement">
        {tmpl_errors}
      </div>
    </div>
  </div>
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  <div class="bx--date-picker bx--date-picker--simple {picker_class}"
      {picker_props}>

    <div class="bx--date-picker-container">
      {tmpl_label}
      {tmpl_element}
    </div>
  </div>
</div>
"""
        return self.format(template, values, context)


class RangeDatePicker(FormNodes):
    """Date Picker component with range inputs.
    """
    NODE_PROPS = ('light',)
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label', 'help', 'picker')
    "Prepare xxx_class and xxx_props values."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('light'), context):
            values['picker_class'].append('bx--date-picker--light')


    def prepare_element_props(self, props, context, bound_field):
        """Prepare html attributes for rendering the form element.
        """
        index = self.bound_fields.index(bound_field)

        props['class'].append('bx--date-picker__input')
        props['pattern'] = r'\d{1,2}/\d{1,2}/\d{4,4}'
        props['placeholder'] = 'mm/dd/yyyy'

        if index:
            props['data-date-picker-input-to'] = ''
        else:
            props['data-date-picker-input-from'] = ''

        if bound_field.errors:
            props['data-invalid'] = ''


    def render_default(self, values, context):
        """Output html of the component.
        """
        has_errors = any(x.errors for x in self.bound_fields)
        if has_errors:
            template = """
<div class="bx--form-item">
  <div data-date-picker data-date-picker-type="range"
      class="bx--date-picker bx--date-picker--range {picker_class}"
      {picker_props}>

    <div class="bx--date-picker-container">
      <label for="{id_0}" class="bx--label {label_class}" {label_props}>
        {label_0}
      </label>
      <div class="bx--date-picker-input__wrapper">
        {tmpl_element_0}
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            data-date-picker-icon="true" class="bx--date-picker__icon"
            width="16" height="16" viewBox="0 0 32 32" aria-hidden="true">
          <path d="M26,4h-4V2h-2v2h-8V2h-2v2H6C4.9,4,4,4.9,4,6v20c0,1.1,0.9,2,2,2h20c1.1,0,2-0.9,2-2V6C28,4.9,27.1,4,26,4z M26,26H6V12h20  V26z M26,10H6V6h4v2h2V6h8v2h2V6h4V10z"></path>
        </svg>
      </div>
      <div class="bx--form-requirement">
        {tmpl_errors_0}
      </div>
    </div>
    <div class="bx--date-picker-container">
      <label for="{id_1}" class="bx--label {label_class}" {label_props}>
        {label_1}
      </label>
      <div class="bx--date-picker-input__wrapper">
        {tmpl_element_1}
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            data-date-picker-icon="true" class="bx--date-picker__icon"
            width="16" height="16" viewBox="0 0 32 32" aria-hidden="true">
          <path d="M26,4h-4V2h-2v2h-8V2h-2v2H6C4.9,4,4,4.9,4,6v20c0,1.1,0.9,2,2,2h20c1.1,0,2-0.9,2-2V6C28,4.9,27.1,4,26,4z M26,26H6V12h20  V26z M26,10H6V6h4v2h2V6h8v2h2V6h4V10z"></path>
        </svg>
      </div>
      <div class="bx--form-requirement">
        {tmpl_errors_1}
      </div>
    </div>
  </div>
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  <div data-date-picker data-date-picker-type="range"
      class="bx--date-picker bx--date-picker--range {picker_class}"
      {picker_props}>

    <div class="bx--date-picker-container">
      <label for="{id_0}" class="bx--label {label_class}" {label_props}>
        {label_0}
      </label>
      <div class="bx--date-picker-input__wrapper">
        {tmpl_element_0}
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            data-date-picker-icon="true" class="bx--date-picker__icon"
            width="16" height="16" viewBox="0 0 32 32" aria-hidden="true">
          <path d="M26,4h-4V2h-2v2h-8V2h-2v2H6C4.9,4,4,4.9,4,6v20c0,1.1,0.9,2,2,2h20c1.1,0,2-0.9,2-2V6C28,4.9,27.1,4,26,4z M26,26H6V12h20  V26z M26,10H6V6h4v2h2V6h8v2h2V6h4V10z"></path>
        </svg>
      </div>
    </div>
    <div class="bx--date-picker-container">
      <label for="{id_1}" class="bx--label {label_class}" {label_props}>
        {label_1}
      </label>
      <div class="bx--date-picker-input__wrapper">
        {tmpl_element_1}
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            data-date-picker-icon="true" class="bx--date-picker__icon"
            width="16" height="16" viewBox="0 0 32 32" aria-hidden="true">
          <path d="M26,4h-4V2h-2v2h-8V2h-2v2H6C4.9,4,4,4.9,4,6v20c0,1.1,0.9,2,2,2h20c1.1,0,2-0.9,2-2V6C28,4.9,27.1,4,26,4z M26,26H6V12h20  V26z M26,10H6V6h4v2h2V6h8v2h2V6h4V10z"></path>
        </svg>
      </div>
    </div>
  </div>
</div>
"""
        return self.format(template, values, context)


components = {
    'DatePicker': DatePicker,
    'RangeDatePicker': RangeDatePicker,
}



"""
Button
======

See: https://www.carbondesignsystem.com/components/button/usage/

Buttons are used to initialize an action. Button labels express what action
will occur when the user interacts with it.

Overview
--------

Buttons are clickable elements that are used to trigger actions. They
communicate calls to action to the user and allow users to interact with pages
in a variety of ways. Button labels express what action will occur when
the user interacts with it.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from .base import Node, modify_svg

class Button(Node):
    """Button component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('disabled', 'variant', 'field', 'small', 'icon_size')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'button'
    "Rendered HTML tag."

    CATCH_PROPS = ('button_props',)

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        variant = self.eval(self.kwargs.get('variant', 'primary'), context)
        values['class'].append(f'bx--btn--{variant}')

        if self.eval(self.kwargs.get('disabled'), context):
            values['props'].append(('disabled', 'disabled'))
            if context.get('button_set'):
                values['class'].append('bx--btn--disabled')

        if self.eval(self.kwargs.get('field'), context):
            values['class'].append('bx--btn--field')

        if self.eval(self.kwargs.get('small'), context):
            values['class'].append('bx--btn--sm')

        if values['label']:
            values['props'].append(('aria-label', values['label']))


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--btn {class}" {props}>
  {child}
  {slot_icon}
</{tag}>
"""
        return self.format(template, values, context)


    def render_slot_icon(self, values, context):
        """Render html of the slot.
        """
        size = self.eval(self.kwargs.get('icon_size', 16), context)
        return modify_svg(values['child'], {
            'focusable': 'false',
            'preserveAspectRatio': 'xMidYMid meet',
            'fill': 'currentColor',
            'style': {
                'width': '%spx' % size,
                'height': '%spx' % size,
            },
            'aria-hidden': 'true',
            'class': 'bx--btn__icon ' + values['class'],
        })


class IconButton(Button):
    """Button icon only component.
    """
    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        super().prepare(values, context)
        values['class'].extend([
                'bx--btn--icon-only',
                'bx--tooltip__trigger',
                'bx--tooltip--a11y',
                'bx--tooltip--bottom',
                'bx--tooltip--align-center'])


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--btn {class}" {props}>
  <span class="bx--assistive-text">{child}</span>
  {slot_icon}
</{tag}>
"""
        return self.format(template, values, context)


class ButtonSet(Node):
    """Button set component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        context['button_set'] = True


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--btn-set {class}" {props}>
  {child}
</{tag}>
"""
        return self.format(template, values)


components = {
    'Button': Button,
    'ButtonSet': ButtonSet,
    'IconButton': IconButton,
}



"""
Grid
====

See: https://the-carbon-components.netlify.app/?nav=grid

""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from .base import Node

class Grid(Node):
    """Grid wrapper component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    MODES = ('default', 'bleed')
    "Available variants."
    NODE_PROPS = ('full_width', 'gap')
    "Extended Template Tag arguments."
    GAP_SIZES = ('narrow', 'condensed')
    "Grid gap sizes."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('full_width'), context):
            values['class'].append('bx--grid--full-width')

        gap = self.eval(self.kwargs.get('gap'), context)
        if gap in self.GAP_SIZES:
            values['class'].append('bx--grid--%s' % gap)


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = '<{tag} class="bx--grid {class}" {props}>{child}</{tag}>'
        return self.format(template, values)


    def render_bleed(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bleed">
  <{tag} class="bx--grid {class}" {props}>{child}</{tag}>
</div>
"""
        return self.format(template, values)


class Row(Node):
    """Grid Row component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = '<{tag} class="bx--row {class}" {props}>{child}</{tag}>'
        return self.format(template, values)


class Column(Node):
    """Grid columns.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    MODES = ('default', 'inner')
    "Available variants."
    COL_SIZES = ('sm', 'md', 'lg', 'xlg', 'max')
    "Column sizes."
    NODE_PROPS = (*COL_SIZES, *['offset_%s' % x for x in COL_SIZES])
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.mode != 'inner':
            values['class'].append('bx--col')

        has_size = False
        for size in self.COL_SIZES:
            width = self.eval(self.kwargs.get(size), context)
            if width:
                has_size = True
                values['class'].append('bx--col-%s-%s' % (size, width))

            width = self.eval(self.kwargs.get('offset_%s' % size), context)
            if width:
                values['class'].append('bx--offset-%s-%s' % (size, width))
        if not has_size:
            values['class'].append('bx--col--auto')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = '<{tag} class="{class}" {props}>{child}</{tag}>'
        return self.format(template, values)


    def render_inner(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="{class}" {props}>
  <div class="outside">
    <div class="inside">
      {child}
    </div>
  </div>
</{tag}>
"""
        return self.format(template, values)


class AspectRatio(Node):
    """Grid aspect ratio.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('ratio',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        ratio = self.eval(self.kwargs.get('ratio'), context)
        if ratio:
            values['class'].append('bx--aspect-ratio--%s' % ratio)


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--aspect-ratio {class}" {props}>
  <div class="bx--aspect-ratio--object">
    {child}
  </div>
</{tag}>
"""
        return self.format(template, values)


components = {
    'Grid': Grid,
    'Row': Row,
    'Col': Column,
    'AspectRatio': AspectRatio,
}



"""
Loading
=======

See: https://www.carbondesignsystem.com/components/loading/usage/

Loading spinners are used when retrieving data or performing slow computations,
and help to notify users that loading is underway.

Overview
--------

Loading spinners are used when retrieving data or performing slow computations.
They notify to the user that their request is being processed. Although they
do not provide details about what is occurring on the back-end, they reassure
the user that their action is being processed.

Use a loading spinner whenever the wait time is anticipated to be longer than
three seconds.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import Node

class Loading(Node):
    """Loading component.
    """
    MODES = ('default', 'overlay', 'small')
    "Available variants."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_loading'] = _("Loading")


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div data-loading class="bx--loading">
  <svg class="bx--loading__svg" viewBox="0 0 100 100">
    <title>{txt_loading}</title>
    <circle class="bx--loading__stroke" cx="50%" cy="50%" r="44" />
  </svg>
</div>
"""
        return self.format(template, values)


    def render_overlay(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bx--loading-overlay">
  <div data-loading class="bx--loading">
    <svg class="bx--loading__svg" viewBox="0 0 100 100">
      <title>{txt_loading}</title>
      <circle class="bx--loading__stroke" cx="50%" cy="50%" r="44" />
    </svg>
  </div>
</div>
"""
        return self.format(template, values)


    def render_small(self, values, context):
        """Output html of the component.
        """
        template = """
<div data-loading class="bx--loading bx--loading--small">
  <svg class="bx--loading__svg" viewBox="0 0 100 100">
    <title>{txt_loading}</title>
    <circle class="bx--loading__background" cx="50%" cy="50%" r="42" />
    <circle class="bx--loading__stroke" cx="50%" cy="50%" r="42" />
  </svg>
</div>
"""
        return self.format(template, values)


components = {
    'Loading': Loading,
}



"""
UI Shell Switcher
=================

See: https://the-carbon-components.netlify.app/?nav=ui-shell

""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import FormNode, Node, clean_attr_value, modify_svg

class UiShellSwitcher(Node):
    """UI Shell switcher panel.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('id',)
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'aside'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--panel--overlay" id="{id}" data-panel-switcher>
  <div class="bx--panel-switcher">
    {child}
  </div>
</{tag}>
"""
        return self.format(template, values, context)


class UiShellSwitcherSearch(FormNode):
    """UI Shell switcher search box.
    """
    NODE_PROPS = ('placeholder',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_clear'] = _("Clear search input")


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].append('bx--search-input')

        placeholder = self.eval(self.kwargs.get('placeholder'), context)
        props['placeholder'] = placeholder or _("Search")


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bx--panel-switcher__search">
  <div class="bx--form-item">
    <div data-search class="bx--search bx--search--sm bx--search--shell"
        role="search">
      {tmpl_label}
      {tmpl_element}
      <svg class="bx--search-magnifier" width="16" height="16"
          viewBox="0 0 16 16">
        <path d="M6.5 12a5.5 5.5 0 1 0 0-11 5.5 5.5 0 0 0 0 11zm4.936-1.27l4.563 4.557-.707.708-4.563-4.558a6.5 6.5 0 1 1 .707-.707z" fill-rule="nonzero" />
      </svg>
      <button class="bx--search-close bx--search-close--hidden"
          title="{txt_clear}" aria-label="{txt_clear}">
        <svg width="16" height="16" viewBox="0 0 16 16" xmlns="http://www.w3.org/2000/svg">
          <path d="M8 6.586L5.879 4.464 4.464 5.88 6.586 8l-2.122 2.121 1.415 1.415L8 9.414l2.121 2.122 1.415-1.415L9.414 8l2.122-2.121-1.415-1.415L8 6.586zM8 16A8 8 0 1 1 8 0a8 8 0 0 1 0 16z" fill-rule="evenodd" />
        </svg>
      </button>
    </div>
  </div>
</div>
"""
        return self.format(template, values, context)


class UiShellSwitcherHeader(Node):
    """UI Shell switcher header.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'p'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--panel-switcher__subheader {class}" {props}>{child}</{tag}>
"""
        return self.format(template, values)


class UiShellSwitcherItem(Node):
    """UI Shell switcher item.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = '<div class="bx--panel-switcher__item">{child}</div>'
        return self.format(template, values)


class UiShellSwitcherMenu(Node):
    """UI Shell switcher menu list.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    DEFAULT_TAG = 'ul'
    "Rendered HTML tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--panel-switcher__panel-list {class}" {props}>{child}</{tag}>
"""
        return self.format(template, values)


class UiShellSwitcherMenuSection(Node):
    """UI Shell switcher submenu.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('menu_label',)
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('overflow',)
    "Prepare xxx_class and xxx_props values."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        menu_label = self.eval(self.kwargs.get('menu_label'), context)
        if menu_label:
            values['overflow_props'].append(('aria-label', menu_label))


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<li class="bx--panel-list__item">
  <a class="bx--panel-link" tabindex="0" href="javascript:void(0)">
    {slot_icon}
    <span class="bx--panel-link__name">{label}</span>
  </a>
  <div data-overflow-menu tabindex="0" class="bx--overflow-menu {overflow_class}" {overflow_props}>
    <svg width="3" height="15" viewBox="0 0 3 15">
      <path d="M0 1.5a1.5 1.5 0 1 1 3 0 1.5 1.5 0 1 1-3 0M0 7.5a1.5 1.5 0 1 1 3 0 1.5 1.5 0 1 1-3 0M0 13.5a1.5 1.5 0 1 1 3 0 1.5 1.5 0 1 1-3 0"></path>
    </svg>
    <ul class="bx--overflow-menu-options bx--overflow-menu--flip" tabindex="-1"
      data-floating-menu-direction="bottom">
      {child}
    </ul>
  </div>
</li>
"""
        return self.format(template, values, context)


    def render_slot_icon(self, values, context):
        """Render html of the slot.
        """
        template = '<div class="bx--panel-switcher__icon">{child}</div>'

        values['child'] = modify_svg(values['child'], {
            'focusable': 'false',
            'preserveAspectRatio': 'xMidYMid meet',
            'fill': 'currentColor',
            'style': {
                'width': '%spx' % 20,
                'height': '%spx' % 20,
            },
            'aria-hidden': 'true',
        })
        return self.format(template, values)


class UiShellSwitcherMenuItem(Node):
    """UI Shell switcher menu item.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('active',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('active'), context):
            values['props'].append(('data-floating-menu-primary-focus', True))


    def render_default(self, values, context):
        """Output html of the component.
        """
        values['cleaned_child'] = clean_attr_value(values['child'])

        template = """
<li class="bx--overflow-menu-options__option bx--overflow__item">
  <button class="bx--overflow-menu-options__btn {class}" title="{cleaned_child}"
      {props}>
    {child}
  </button>
</li>
"""
        return self.format(template, values)


components = {
    'UiSwitcher': UiShellSwitcher,
    'UiSwitcherSearch': UiShellSwitcherSearch,
    'UiSwitcherHeader': UiShellSwitcherHeader,
    'UiSwitcherItem': UiShellSwitcherItem,
    'UiSwitcherMenu': UiShellSwitcherMenu,
    'UiSwitcherMenuSection': UiShellSwitcherMenuSection,
    'UiSwitcherMenuItem': UiShellSwitcherMenuItem,
}



"""
Code Snippet
============

See: https://www.carbondesignsystem.com/components/code-snippet/usage/

Code snippets are strings or small blocks of reusable code that can be copied
and inserted in a code file.

Overview
--------

There are three different variants of code snippets to help cater to varied
line length use cases—inline, single line, and multi-line.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import Node

class CodeSnippet(Node):
    """Code Snippet component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    MODES = ('inline', 'single', 'multi')
    "Available variants."
    NODE_PROPS = ('light',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_copied'] = _("Copied!")
        values['txt_copy'] = _("Copy code")
        values['txt_show_more'] = _("Show more")
        values['txt_show_more_icon'] = _("Show more icon")
        values['txt_show_less'] = _("Show less")

        if self.eval(self.kwargs.get('light'), context):
            values['class'].append('bx--snippet--light')


    def render_multi(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--snippet bx--snippet--multi {class}" data-code-snippet {props}>
  <div class="bx--snippet-container" aria-label="{label}">
    <pre><code>{child}</code></pre>
  </div>
  <button data-copy-btn class="bx--copy-btn" type="button" tabindex="0">
    <span class="bx--assistive-text bx--copy-btn__feedback">{txt_copied}</span>
    <svg focusable="false" preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg" fill="currentColor"
        class="bx--snippet__icon" width="16" height="16" viewBox="0 0 32 32"
        aria-hidden="true">
      <path d="M28,10V28H10V10H28m0-2H10a2,2,0,0,0-2,2V28a2,2,0,0,0,2,2H28a2,2,0,0,0,2-2V10a2,2,0,0,0-2-2Z"></path>
      <path d="M4,18H2V4A2,2,0,0,1,4,2H18V4H4Z"></path>
    </svg>
  </button>
  <button class="bx--btn bx--btn--ghost bx--btn--sm bx--snippet-btn--expand"
      type="button">
    <span class="bx--snippet-btn--text" data-show-more-text="{txt_show_more}"
        data-show-less-text="{txt_show_less}">
      {txt_show_more}
    </span>
    <svg focusable="false" preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg" fill="currentColor"
        aria-label="{txt_show_more_icon}"
        class="bx--icon-chevron--down bx--snippet__icon" width="16" height="16"
        viewBox="0 0 16 16" role="img">
      <path d="M8 11L3 6 3.7 5.3 8 9.6 12.3 5.3 13 6z"></path>
    </svg>
  </button>
</div>
"""
        return self.format(template, values)


    def render_single(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--snippet bx--snippet--single {class}" {props}>
  <div tabindex="0"  class="bx--snippet-container" aria-label="{label}">
    <pre><code>{child}</code></pre>
  </div>
  <button data-copy-btn class="bx--copy-btn" type="button" tabindex="0">
    <span class="bx--assistive-text bx--copy-btn__feedback">{txt_copied}</span>
    <svg focusable="false" preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg" fill="currentColor"
        class="bx--snippet__icon" width="16" height="16" viewBox="0 0 32 32"
        aria-hidden="true">
      <path d="M28,10V28H10V10H28m0-2H10a2,2,0,0,0-2,2V28a2,2,0,0,0,2,2H28a2,2,0,0,0,2-2V10a2,2,0,0,0-2-2Z"></path>
      <path d="M4,18H2V4A2,2,0,0,1,4,2H18V4H4Z"></path>
    </svg>
  </button>
</{tag}>
"""
        return self.format(template, values)


    def render_inline(self, values, context):
        """Output html of the component.
        """
        template = """
<button data-copy-btn type="button"
    class="bx--snippet bx--snippet--inline {class}"
    aria-label="{txt_copy}" tabindex="0" {props}>
  <code>{child}</code>
  <span class="bx--assistive-text bx--copy-btn__feedback">{txt_copied}</span>
</button>
"""
        return self.format(template, values)


components = {
    'Code': CodeSnippet,
}



"""
Inline Loading
==============

See: https://www.carbondesignsystem.com/components/inline-loading/usage/

The inline loading component provides visual feedback that data is being
processed.

Overview
--------

Inline loading spinners are used when performing actions. They notify to
the user that their request is being processed. Although they do not provide
details about what is occurring on the back-end, they reassure the user that
their action is being processed.

Common actions that benefit from inline loading include any create, update,
or delete actions that may have a lot of data to process. It can be used in
a table, after a primary or secondary button click, or even in a modal.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import Node

class InlineLoading(Node):
    """Inline Loading component.
    """

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_loading'] = _("Loading data...")
        values['txt_loaded'] = _("Data loaded.")
        values['txt_failed'] = _("Loading data failed.")


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div data-inline-loading class="bx--inline-loading" role="alert"
    aria-live="assertive">
  <div class="bx--inline-loading__animation">
    <div data-inline-loading-spinner class="bx--loading bx--loading--small">
      <svg class="bx--loading__svg" viewBox="0 0 100 100">
        <circle class="bx--loading__background" cx="50%" cy="50%" r="42" />
        <circle class="bx--loading__stroke" cx="50%" cy="50%" r="42" />
      </svg>
    </div>
    <svg focusable="false" preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg" fill="currentColor"
        class="bx--inline-loading__checkmark-container" hidden=""
        data-inline-loading-finished="" width="16" height="16"
        viewBox="0 0 16 16" aria-hidden="true">
      <path d="M8,1C4.1,1,1,4.1,1,8c0,3.9,3.1,7,7,7s7-3.1,7-7C15,4.1,11.9,1,8,1z M7,11L4.3,8.3l0.9-0.8L7,9.3l4-3.9l0.9,0.8L7,11z"></path>
      <path d="M7,11L4.3,8.3l0.9-0.8L7,9.3l4-3.9l0.9,0.8L7,11z" data-icon-path="inner-path" opacity="0"></path>
    </svg>
    <svg focusable="false" preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg" fill="currentColor"
        class="bx--inline-loading--error" hidden="" data-inline-loading-error=""
        width="20" height="20" viewBox="0 0 32 32" aria-hidden="true">
      <path d="M2,16H2A14,14,0,1,0,16,2,14,14,0,0,0,2,16Zm23.15,7.75L8.25,6.85a12,12,0,0,1,16.9,16.9ZM8.24,25.16A12,12,0,0,1,6.84,8.27L23.73,25.16a12,12,0,0,1-15.49,0Z"></path>
    </svg>
  </div>
  <p data-inline-loading-text-active class="bx--inline-loading__text">
    {txt_loading}
  </p>
  <p data-inline-loading-text-finished hidden class="bx--inline-loading__text">
    {txt_loaded}
  </p>
  <p data-inline-loading-text-error hidden class="bx--inline-loading__text">
    {txt_failed}
  </p>
</div>
"""
        return self.format(template, values)


components = {
    'InlineLoading': InlineLoading,
}



"""
Dropdown
========

See: https://www.carbondesignsystem.com/components/dropdown/usage/

This module is currently disabled, it's a weird component that receive input,
but there is no html input element. Wait and see its progress.

Dropdowns present a list of options from which a user can select one option,
or several. A selected option can represent a value in a form, or can be used
as an action to filter or sort existing content.

Overview
--------

There are three different variants of dropdowns that support various kinds of
functionality—dropdown, multiselect, and combo box.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from .base import Node, clean_attr_value, modify_svg

class Dropdown(Node):
    """Dropdown component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('help', 'errors')
    "Named children."
    NODE_PROPS = ('id', 'value', 'disabled', 'up', 'light', 'inline')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label', 'help', 'wrapper', 'dropdown')
    "Prepare xxx_class and xxx_props values."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['value'] = self.eval(self.kwargs.get('value', ''), context)

        if self.eval(self.kwargs.get('disabled', False), context):
            values['class'].append('bx--dropdown--disabled')
            values['props'].append(('disabled', 'disabled'))
            values['label_class'].append('bx--label--disabled')
            values['label_props'].append(('aria-disabled', 'true'))
            values['help_class'].append('bx--form__helper-text--disabled')
            values['help_props'].append(('aria-disabled', 'true'))
            context['dropdown_disabled'] = True

        if self.eval(self.kwargs.get('up', False), context):
            values['class'].append('bx--dropdown--up')

        if self.eval(self.kwargs.get('light', False), context):
            values['class'].append('bx--dropdown--light')

        if self.eval(self.kwargs.get('inline', False), context):
            values['wrapper_class'].append('bx--dropdown__wrapper--inline')
            values['dropdown_props'].append(('data-dropdown-type', 'inline'))
            values['class'].append('bx--dropdown--inline')


    def render_default(self, values, context):
        """Output html of the component.
        """
        if 'errors' in self.slots:
            template = """
<div class="bx--form-item">
  <div class="bx--dropdown__wrapper {wrapper_class}">
    <span id="label-{id}" class="bx--label {label_class}" {label_props}>
      {label}
    </span>
    <div data-dropdown data-value
        class="bx--dropdown bx--dropdown--invalid {class}" data-invalid
        {dropdown_props}>
      <button class="bx--dropdown-text" aria-haspopup="true"
          aria-expanded="false" aria-controls="menu-{id}"
          aria-labelledby="label-{id} value-{id}" type="button" {props}>
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            class="bx--dropdown__invalid-icon" width="16" height="16"
            viewBox="0 0 16 16" aria-hidden="true">
          <path d="M8,1C4.2,1,1,4.2,1,8s3.2,7,7,7s7-3.1,7-7S11.9,1,8,1z M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2   c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z">
          </path><path d="M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8 c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z" data-icon-path="inner-path" opacity="0"></path>
        </svg>
        <span class="bx--dropdown-text__inner" id="value-{id}">
          {value}
        </span>
        <span class="bx--dropdown__arrow-container">
          <svg focusable="false" preserveAspectRatio="xMidYMid meet"
              xmlns="http://www.w3.org/2000/svg" fill="currentColor"
              class="bx--dropdown__arrow" width="16" height="16"
              viewBox="0 0 16 16" aria-hidden="true">
            <path d="M8 11L3 6 3.7 5.3 8 9.6 12.3 5.3 13 6z"></path>
          </svg>
        </span>
      </button>
      <ul class="bx--dropdown-list" id="menu-{id}" role="menu" tabindex="-1"
          aria-hidden="true" aria-labelledby="label-{id}">
        {child}
      </ul>
    </div>
    {slot_help}
    <div class="bx--form-requirement">
      {slot_errors}
    </div>
  </div>
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  <div class="bx--dropdown__wrapper {wrapper_class}">
    <span id="label-{id}" class="bx--label {label_class}" {label_props}>
      {label}
    </span>
    <div data-dropdown data-value class="bx--dropdown {class}" {dropdown_props}>
      <button class="bx--dropdown-text" aria-haspopup="true"
          aria-expanded="false" aria-controls="menu-{id}"
          aria-labelledby="label-{id} value-{id}" type="button" {props}>
        <span class="bx--dropdown-text__inner" id="value-{id}">
          {value}
        </span>
        <span class="bx--dropdown__arrow-container">
          <svg focusable="false" preserveAspectRatio="xMidYMid meet"
              xmlns="http://www.w3.org/2000/svg" fill="currentColor"
              class="bx--dropdown__arrow" width="16" height="16"
              viewBox="0 0 16 16" aria-hidden="true">
            <path d="M8 11L3 6 3.7 5.3 8 9.6 12.3 5.3 13 6z"></path>
          </svg>
        </span>
      </button>
      <ul class="bx--dropdown-list" id="menu-{id}" role="menu" tabindex="-1"
          aria-hidden="true" aria-labelledby="label-{id}">
        {child}
      </ul>
    </div>
    {slot_help}
  </div>
</div>
"""
        return self.format(template, values, context)


    def render_slot_help(self, values, context):
        """Render html of the slot.
        """
        template = """
<div class="bx--form__helper-text {class}" {props}>{child}</div>
"""
        return self.format(template, values, context)


class DropdownItem(Node):
    """Dropdown item component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('icon',)
    "Named children."
    NODE_PROPS = ('active', 'value')
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['value'] = self.eval(self.kwargs.get('value', ''), context)

        if self.eval(self.kwargs.get('active'), context):
            values['props'].append(('aria-checked', 'true'))
        else:
            values['props'].append(('aria-checked', 'false'))
            values['props'].append(('tabindex', '-1'))

        # Note: remove `tabindex` from disabled dropdown lists
        if context.get('dropdown_disabled'):
            values['props'].append(('tabindex', '-1'))


    def render_default(self, values, context):
        """Output html of the component.
        """
        values['cleaned_child'] = clean_attr_value(values['child'])

        template = """
<li data-option data-value="{value}" class="bx--dropdown-item {class}"
    title="{cleaned_child}">
  <a class="bx--dropdown-link" role="menuitemradio" id="item-{id}" {props}>
    {child}
  </a>
  {slot_icon}
</li>
"""
        return self.format(template, values, context)


    def render_slot_icon(self, values, context):
        """Render html of the slot.
        """
        return modify_svg(values['child'], {
            'focusable': 'false',
            'preserveAspectRatio': 'xMidYMid meet',
            'fill': 'currentColor',
            'class': 'bx--list-box__menu-item__selected-icon ' +\
                values['class'],
            'style': {
                'will-change': 'transform',
                'width': '%spx' % 16,
                'height': '%spx' % 16,
            },
            'aria-hidden': 'true',
        })


components = {
    'Dropdown': Dropdown,
    'DropdownItem': DropdownItem,
}



"""
Checkbox
========

See: https://www.carbondesignsystem.com/components/checkbox/usage/

Checkboxes are used when there are multiple items to select in a list. Users
can select zero, one, or any number of items.

Overview
--------

Checkboxes are used for multiple choices, not for mutually exclusive choices.
Each checkbox works independently from other checkboxes in the list, therefore
checking an additional box does not affect any other selections.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from .base import ChoiceFormNode

class CheckBox(ChoiceFormNode):
    """Checkbox component.
    """
    MODES = ('default', 'inside')
    "Available variants."
    NODE_PROPS = ('mixed',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('mixed'), context):
            values['props'].append(('aria-checked', 'mixed'))

            if self.mode == 'inside':
                values['label_props'].append(('data-contained-checkbox-state',
                        'mixed'))


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].append('bx--checkbox')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bx--form-item bx--checkbox-wrapper">
  {tmpl_element}
  {tmpl_label}
</div>
"""
        return self.format(template, values, context)


    def render_inside(self, values, context,):
        """Output html of the component.
        """
        template = """
<div class="bx--form-item bx--checkbox-wrapper">
  <label for="{id}" class="bx--checkbox-label {label_class}" {label_props}>
    {tmpl_element}
    {label}
  </label>
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_label(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if not values['label']:
            return ''
        template = """
<label for="{id}" class="bx--checkbox-label {label_class}" {label_props}>
  {label}
</label>
"""
        return self.format(template, values)


components = {
    'CheckBox': CheckBox,
}



"""
Toggle
======

See: https://www.carbondesignsystem.com/components/toggle/usage/

A toggle is used to quickly switch between two possible states. They are
commonly used for “on/off” switches.

Overview
--------

Toggle is a control that is used to quickly switch between two possible states.
Toggles are only used for these binary actions that occur immediately after
the user “flips the switch”. They are commonly used for “on/off” switches.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import FormNode

class Toggle(FormNode):
    """Toggle component.
    """
    MODES = ('default', 'nolabel')
    "Extended Template Tag arguments."
    NODE_PROPS = ('small',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_off'] = _("Off")
        values['txt_on'] = _("On")


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].append('bx--toggle-input')

        if self.eval(self.kwargs.get('small'), context):
            props['class'].append('bx--toggle-input--small')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bx--form-item">
  {tmpl_element}
  <label class="bx--toggle-input__label" for="{id}">
    {label}
    <span class="bx--toggle__switch">
      {tmpl_icon}
      <span class="bx--toggle__text--off" aria-hidden="true">{txt_off}</span>
      <span class="bx--toggle__text--on" aria-hidden="true">{txt_on}</span>
    </span>
  </label>
  {tmpl_help}
</div>
"""
        return self.format(template, values, context)


    def render_nolabel(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bx--form-item">
  {tmpl_element}
  <label class="bx--toggle-input__label" for="{id}" aria-label="{label}">
    <span class="bx--toggle__switch">
      {tmpl_icon}
      <span class="bx--toggle__text--off" aria-hidden="true">{txt_off}</span>
      <span class="bx--toggle__text--on" aria-hidden="true">{txt_on}</span>
    </span>
  </label>
  {tmpl_help}
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_icon(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if self.eval(self.kwargs.get('small'), context):
            return """
<svg class="bx--toggle__check" width="6px" height="5px" viewBox="0 0 6 5">
  <path d="M2.2 2.7L5 0 6 1 2.2 5 0 2.7 1 1.5z" />
</svg>
"""
        return ''


components = {
    'Toggle': Toggle,
}



"""
Link
====

See: https://www.carbondesignsystem.com/components/link/usage/

Links are used as navigational elements. They may appear on their own, within
a sentence or paragraph, or directly following the content.

Overview
--------

Links are used as navigational elements and can be used on their own or inline
with text. They provide a lightweight option for navigation but like other
interactive elements, too many links will clutter a page and make it difficult
for users to identify their next steps. This is especially true for inline
links, which should be used sparingly.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from .base import Node

class Link(Node):
    """Link component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('visited', 'disabled', 'inline')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'a'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        is_a = values['tag'] == 'a'

        if is_a:
            values['class'].append('bx--link')

        if self.eval(self.kwargs.get('visited', False), context):
            values['class'].append('bx--link--visited')

        if self.eval(self.kwargs.get('disabled', False), context):
            if is_a:
                values['props'].append(('aria-disabled', 'true'))
            else:
                values['class'].append('bx--link--disabled')

        if self.eval(self.kwargs.get('inline', False), context):
            values['class'].append('bx--link--inline')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = '<{tag} class="{class}" {props}>{child}</{tag}>'
        return self.format(template, values)


components = {
    'Link': Link,
}



"""
Structured List
===============

See: https://www.carbondesignsystem.com/components/structured-list/usage/

Structured lists group content that is similar or related, such as terms and
definitions.

Overview
--------

Structured list displays a simple list with a considerable amount of items of
read-only values. It helps organize and present grouped information into
logical and scannable patterns. The content within a list can be stacked to
create hierarchy within the data.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from .base import Node, FormNode

class StructuredList(Node):
    """Structured List component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('header',)
    "Named children."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<section class="bx--structured-list">
  <div class="bx--structured-list-thead">
    <div class="bx--structured-list-row bx--structured-list-row--header-row">
      {slot_header}
    </div>
  </div>
  <div class="bx--structured-list-tbody">
    {child}
  </div>
</section>
"""
        return self.format(template, values, context)


class StructuredListSelection(FormNode):
    """Structured List Selection component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('header',)
    "Named children."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        context['bound_field'] = self.bound_field
        context['bound_choices'] = self.choices()


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<section class="bx--structured-list bx--structured-list--selection"
    data-structured-list>
  <div class="bx--structured-list-thead">
    <div class="bx--structured-list-row bx--structured-list-row--header-row">
      {slot_header}
    </div>
  </div>
  <div class="bx--structured-list-tbody">
    {child}
  </div>
</section>
"""
        return self.format(template, values, context)


class StructuredListTh(Node):
    """Structured List column header component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bx--structured-list-th {class}" {props}>{child}</div>
"""
        return self.format(template, values)


class StructuredListTd(Node):
    """Structured List column header component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('nowrap',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('nowrap'), context):
            values['class'].append('bx--structured-list-content--nowrap')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div class="bx--structured-list-td {class}" {props}>
  {child}
</div>
"""
        return self.format(template, values)


class StructuredListRow(Node):
    """Structured List row component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('value',)
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label',)
    "Prepare xxx_class and xxx_props values."

    bound_field = None

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        self.bound_field = context.get('bound_field')

        if self.bound_field:
            values['name'] = self.bound_field.name
            values['value'] = value = self.eval(self.kwargs.get('value'),
                    context)

            selected = self.bound_field.value()
            if isinstance(selected, str):
                selected = [selected]
            if selected and value in selected:
                values['label_class'].append(
                        'bx--structured-list-row--selected')
                values['props'].append(('checked', True))

            choices = context['bound_choices']
            for _, val, txt in choices:
                if val == value:
                    values['label_props'].append(('aria-label', txt))
                    values['props'].append(('title', txt))
                    break


    def render_default(self, values, context):
        """Output html of the component.
        """
        if self.bound_field:
            template = """
<label class="bx--structured-list-row {label_class}" tabindex="0" {label_props}>
  {child}
  <input tabindex="-1" class="bx--structured-list-input {class}" value="{value}"
      type="radio" name="{name}" {props}>
  <div class="bx--structured-list-td">
    <svg focusable="false" preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg" fill="currentColor"
        class="bx--structured-list-svg" width="16" height="16"
        viewBox="0 0 16 16" aria-hidden="true">
      <path d="M8,1C4.1,1,1,4.1,1,8c0,3.9,3.1,7,7,7s7-3.1,7-7C15,4.1,11.9,1,8,1z M7,11L4.3,8.3l0.9-0.8L7,9.3l4-3.9l0.9,0.8L7,11z"></path>
      <path d="M7,11L4.3,8.3l0.9-0.8L7,9.3l4-3.9l0.9,0.8L7,11z" data-icon-path="inner-path" opacity="0"></path>
    </svg>
  </div>
</label>
"""
        else:
            template = """
<div class="bx--structured-list-row">
  {child}
</div>
        """
        return self.format(template, values)


components = {
    'Sl': StructuredList,
    'SlSelect': StructuredListSelection,
    'SlTh': StructuredListTh,
    'SlTr': StructuredListRow,
    'SlTd': StructuredListTd,
}



"""
Select
======

See: https://www.carbondesignsystem.com/components/select/usage/

The select component allows users to choose one option from a list. It is used
in forms for users to submit data.

Overview
--------

Select is a type of input that is used in forms, where a user is submitting
data and chooses one option from a list.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import FormNode

class Select(FormNode):
    """Select component.
    """
    MODES = ('default', 'inline')
    "Available variants."
    NODE_PROPS = ('light',)
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label', 'help', 'select')
    "Prepare xxx_class and xxx_props values."

    required = False

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_choose'] = _("Choose an option")

        values['props'].append(('id', values['id']))
        values['props'].append(('name', self.bound_field.name))

        self.required = self.bound_field.field.required and\
                self.bound_field.form.use_required_attribute
        if self.required:
            values['props'].append(('required', True))

        if self.eval(self.kwargs.get('disabled'), context):
            values['props'].append(('disabled', True))
            values['label_class'].append('bx--label--disabled')
            values['select_class'].append('bx--select--disabled')

        if self.eval(self.kwargs.get('light'), context):
            values['select_class'].append('bx--select--light')


    def render_default(self, values, context):
        """Output html of the component.
        """
        if self.bound_field.errors:
            template = """
<div class="bx--form-item">
  <div class="bx--select bx--select--invalid {select_class}" {select_props}>
    {tmpl_label}
    <div class="bx--select-input__wrapper" data-invalid>
      <select class="bx--select-input {class}" {props}>
        {tmpl_items}
      </select>
      <svg focusable="false" preserveAspectRatio="xMidYMid meet"
          xmlns="http://www.w3.org/2000/svg" fill="currentColor"
          class="bx--select__arrow" width="16" height="16" viewBox="0 0 16 16"
          aria-hidden="true">
        <path d="M8 11L3 6 3.7 5.3 8 9.6 12.3 5.3 13 6z"></path>
      </svg>
      {tmpl_icon_error}
    </div>
    <div class="bx--form-requirement">
      {tmpl_errors}
    </div>
    {tmpl_help}
  </div>
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  <div class="bx--select {select_class}" {select_props}>
    {tmpl_label}
    <div class="bx--select-input__wrapper">
      <select class="bx--select-input {class}" {props}>
        {tmpl_items}
      </select>
      <svg focusable="false" preserveAspectRatio="xMidYMid meet"
          xmlns="http://www.w3.org/2000/svg" fill="currentColor"
          class="bx--select__arrow" width="16" height="16" viewBox="0 0 16 16"
          aria-hidden="true">
        <path d="M8 11L3 6 3.7 5.3 8 9.6 12.3 5.3 13 6z"></path>
      </svg>
    </div>
    {tmpl_help}
  </div>
</div>
"""
        return self.format(template, values, context)


    def render_inline(self, values, context):
        """Output html of the component.
        """
        if self.bound_field.errors:
            template = """
<div class="bx--form-item">
  <div class="bx--select bx--select--inline bx--select--invalid {select_class}"
      {select_props}>
    {tmpl_label}
    <div class="bx--select-input--inline__wrapper">
      <div class="bx--select-input__wrapper" data-invalid>
        <select class="bx--select-input {class}" {props}>
          {tmpl_items}
        </select>
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            class="bx--select__arrow" width="16" height="16" viewBox="0 0 16 16"
            aria-hidden="true">
          <path d="M8 11L3 6 3.7 5.3 8 9.6 12.3 5.3 13 6z"></path>
        </svg>
        {tmpl_icon_error}
      </div>
      <div class="bx--form-requirement">
        {tmpl_errors}
      </div>
    </div>
    {tmpl_help}
  </div>
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  <div class="bx--select bx--select--inline {select_class}" {select_props}>
    {tmpl_label}
    <div class="bx--select-input--inline__wrapper">
      <div class="bx--select-input__wrapper">
        <select class="bx--select-input {class}" {props}>
          {tmpl_items}
        </select>
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            class="bx--select__arrow" width="16" height="16" viewBox="0 0 16 16"
            aria-hidden="true">
          <path d="M8 11L3 6 3.7 5.3 8 9.6 12.3 5.3 13 6z"></path>
        </svg>
      </div>
    </div>
    {tmpl_help}
  </div>
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_icon_error(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if self.eval(self.kwargs.get('disabled'), context):
            return ''
        return """
<svg focusable="false" preserveAspectRatio="xMidYMid meet"
    xmlns="http://www.w3.org/2000/svg" fill="currentColor"
    class="bx--select__invalid-icon" width="16" height="16"
    viewBox="0 0 16 16" aria-hidden="true">
  <path d="M8,1C4.2,1,1,4.2,1,8s3.2,7,7,7s7-3.1,7-7S11.9,1,8,1z M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2	c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z"></path>
  <path d="M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8	c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z" data-icon-path="inner-path" opacity="0"></path>
</svg>
"""


    def render_tmpl_items(self, values, context):
        """Dynamically render a part of the component's template.
        """
        group_begin_tmpl = """
<optgroup class="bx--select-optgroup" label="{label}">
"""
        group_end_tmpl = '</optgroup>'
        item_tmpl = """
<option class="bx--select-option" value="{value}" {props}>{child}</option>
"""
        items = []
        items.append(item_tmpl.format(value='', child=values['txt_choose'],
                props=''))

        current_group = None
        for group, value, label in self.choices():
            if group != current_group:
                if current_group:
                    items.append(group_end_tmpl)
                current_group = group
                items.append(group_begin_tmpl.format(label=group))

            options = {'value': value, 'child': label}
            props = []
            if self.bound_value and value in self.bound_value:
                props.append('selected')
            options['props'] = ' '.join(props)
            items.append(self.format(item_tmpl, options))
        if current_group:
            items.append(group_end_tmpl)

        return '\n'.join(items)


components = {
    'Select': Select,
}



"""
Text Area
=========

See: https://the-carbon-components.netlify.app/?nav=text-area

""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.forms.widgets import Textarea
#-
from .base import FormNode

class TextArea(FormNode):
    """Text Area component.
    """
    NODE_PROPS = ('light',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        widget = self.bound_field.field.widget
        if not isinstance(widget, Textarea):
            self.bound_field.field.widget = Textarea(widget.attrs)

        if self.eval(self.kwargs.get('disabled'), context):
            values['label_class'].append('bx--label--disabled')
            values['help_class'].append('bx--form__helper-text--disabled')


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].extend([
                'bx--text-area',
                'bx--text-area--v2'])

        if self.eval(self.kwargs.get('light'), context):
            props['class'].append('bx--text-area--light')

        if self.bound_field.errors:
            props['class'].append('bx--text-area--invalid')


    def render_default(self, values, context):
        """Output html of the component.
        """
        if self.bound_field.errors:
            template = """
<div class="bx--form-item">
  {tmpl_label}
  <div class="bx--text-area__wrapper" data-invalid>
    {tmpl_icon_invalid}
    {tmpl_element}
  </div>
  <div class="bx--form-requirement">
    {tmpl_errors}
  </div>
  {tmpl_help}
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  {tmpl_label}
  <div class="bx--text-area__wrapper">
    {tmpl_element}
  </div>
  {tmpl_help}
</div>

"""
        return self.format(template, values, context)


    def render_tmpl_icon_invalid(self, values, context):
        """Dynamically render a part of the component's template.
        """
        return """
<svg focusable="false" preserveAspectRatio="xMidYMid meet"
    xmlns="http://www.w3.org/2000/svg" fill="currentColor"
    class="bx--text-area__invalid-icon" width="16" height="16"
    viewBox="0 0 16 16" aria-hidden="true">
  <path d="M8,1C4.2,1,1,4.2,1,8s3.2,7,7,7s7-3.1,7-7S11.9,1,8,1z M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2	c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z"></path>
  <path d="M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8	c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z" data-icon-path="inner-path" opacity="0"></path>
</svg>
"""


components = {
    'TextArea': TextArea,
}



"""
UI Shell
========

See: https://the-carbon-components.netlify.app/?nav=ui-shell

""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import Node, clean_attr_value, modify_svg

class UiShell(Node):
    """UI Shell component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('title', 'title_prefix', 'navigation', 'links', 'actions',
            'sidenav', 'switcher')
    "Named children."
    NODE_PROPS = ('id', 'href', 'label_prefix')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'header'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_skip_menu'] = _("Skip to main content")
        values['txt_open_menu'] = _("Open menu")
        values['txt_close_menu'] = _("Close menu")

        values['href'] = self.eval(self.kwargs.get('href', '#'), context)

        if 'label_prefix' in self.kwargs:
            values['long_label'] = '%s %s' % (
                    self.eval(self.kwargs.get('label_prefix'), context),
                    values['label'])
        else:
            values['long_label'] = values['label']


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--header {class}" role="banner" aria-label="{long_label}"
    data-header {props}>
  <a class="bx--skip-to-content" href="#main-content" tabindex="0">
    {txt_skip_menu}
  </a>
  {tmpl_hamburger}
  <a class="bx--header__name" href="{href}" title="">
    {tmpl_title}
  </a>
  {slot_links}
  {slot_actions}
</{tag}>

{slot_sidenav}
{slot_switcher}
{slot_navigation}
<div class="bx--content">{child}</div>
"""
        return self.format(template, values, context)


    def render_slot_navigation(self, values, context):
        """Render html of the slot.
        """
        template = """
<div class="bx--navigation {class}" id="navigation-menu-{id}"
    hidden data-navigation-menu {props}>
  {child}
</div>
"""
        return self.format(template, values)


    def render_slot_links(self, values, context):
        """Render html of the slot.
        """
        template = """
<nav class="bx--header__nav {class}" aria-label="{label}" data-header-nav>
  <ul class="bx--header__menu-bar" aria-label="{label}" {props}>
    {child}
  </ul>
</nav>
"""
        return self.format(template, values)


    def render_slot_actions(self, values, context):
        """Render html of the slot.
        """
        template = """
<div class="bx--header__global {class}" {props}>
  {child}
</div>
"""
        return self.format(template, values)


    def render_slot_title_prefix(self, values, context):
        """Render html of the slot.
        """
        template = """
<span class="bx--header__name--prefix {class}" {props}>{child} &nbsp;</span>
"""
        return self.format(template, values)


    def render_tmpl_title(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if 'title' in self.slots:
            title = '{slot_title}'
        else:
            title = '{label}'
        if 'title_prefix' in self.slots:
            template = '{slot_title_prefix} ' + title
        else:
            template = title
        return self.format(template, values, context)


    def render_tmpl_hamburger(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if 'navigation' not in self.slots:
            return ''

        template = """
<button class="bx--header__menu-trigger bx--header__action"
    aria-label="{txt_open_menu}" title="{txt_open_menu}"
    data-navigation-menu-panel-label-expand="{txt_open_menu}"
    data-navigation-menu-panel-label-collapse="{txt_close_menu}"
    data-navigation-menu-target="#navigation-menu-{id}">
  <svg focusable="false" preserveAspectRatio="xMidYMid meet"
      xmlns="http://www.w3.org/2000/svg" fill="currentColor"
      aria-hidden="true" class="bx--navigation-menu-panel-collapse-icon"
      width="20" height="20" viewBox="0 0 32 32">
    <path d="M24 9.4L22.6 8 16 14.6 9.4 8 8 9.4 14.6 16 8 22.6 9.4 24 16 17.4 22.6 24 24 22.6 17.4 16 24 9.4z"></path>
  </svg>
  <svg focusable="false" preserveAspectRatio="xMidYMid meet"
      xmlns="http://www.w3.org/2000/svg" fill="currentColor"
      aria-hidden="true" class="bx--navigation-menu-panel-expand-icon"
      width="20" height="20" viewBox="0 0 20 20">
    <path d="M2 14.8H18V16H2zM2 11.2H18V12.399999999999999H2zM2 7.6H18V8.799999999999999H2zM2 4H18V5.2H2z"></path>
  </svg>
</button>
"""
        return self.format(template, values)


class Link(Node):
    """Navigation link.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('submenu',)
    "Named children."
    NODE_PROPS = ('expanded',)
    "Extended Template Tag arguments."

    is_submenu = False

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        self.is_submenu = context.get('navlink_submenu')
        if self.is_submenu:
            values['props'].append(('tabindex', '-1'))
        else:
            values['props'].append(('tabindex', '0'))

        if 'submenu' in self.slots:
            context['navlink_submenu'] = True

        if 'submenu' in self.slots:
            if self.eval(self.kwargs.get('expanded'), context):
                values['props'].append(('aria-expanded', 'true'))
            else:
                values['props'].append(('aria-expanded', 'false'))


    def render_default(self, values, context):
        """Output html of the component.
        """
        if 'submenu' in self.slots:
            values['cleaned_child'] = clean_attr_value(values['child'])
            template = """
<li class="bx--header__submenu" data-header-submenu>
  <a class="bx--header__menu-item bx--header__menu-title {class}"
      aria-haspopup="true" {props}>
    {child}
    <svg class="bx--header__menu-arrow" width="12" height="7"
        aria-hidden="true">
      <path d="M6.002 5.55L11.27 0l.726.685L6.003 7 0 .685.726 0z" />
    </svg>
  </a>
  {slot_submenu}
</li>
"""
            return self.format(template, values, context)

        if self.is_submenu:
            template = """
<li role="none">
  <a class="bx--header__menu-item {class}" {props}>
    <span class="bx--text-truncate--end">
      {child}
    </span>
  </a>
</li>
"""
        else:
            template = """
<li>
  <a class="bx--header__menu-item {class}" {props}>
    {child}
  </a>
</li>
"""
        return self.format(template, values)


    def render_slot_submenu(self, values, context):
        """Render html of the slot.
        """
        template = """
<ul class="bx--header__menu {class}" aria-label="{cleaned_child}" {props}>
  {child}
</ul>
"""
        return self.format(template, values)


class Action(Node):
    """Topbar action buttons.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('svg_open', 'svg_close')
    "Named children."
    NODE_PROPS = ('target',)
    "Extended Template Tag arguments."
    REQUIRED_PROPS = ('target',)
    "Will raise Exception if not set."
    DEFAULT_TAG = 'button'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_close_menu'] = _("Close menu")
        values['target'] = self.eval(self.kwargs['target'], context)


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--header__menu-trigger bx--header__action {class}"
    aria-label="{label}" title="{label}"
    data-navigation-menu-panel-label-expand="{label}"
    data-navigation-menu-panel-label-collapse="{txt_close_menu}"
    data-switcher-target="#{target}" {props}>
  {slot_svg_open}
  {slot_svg_close}
</{tag}>
"""
        return self.format(template, values, context)


    def render_slot_svg_open(self, values, context):
        """Render html of the slot.
        """
        return modify_svg(values['child'], {
            'focusable': 'false',
            'preserveAspectRatio': 'xMidYMid meet',
            'fill': 'currentColor',
            'style': {
                'width': '%spx' % 20,
                'height': '%spx' % 20,
            },
            'aria-hidden': 'true',
            'class': 'bx--navigation-menu-panel-expand-icon',
        })


    def render_slot_svg_close(self, values, context):
        """Render html of the slot.
        """
        return modify_svg(values['child'], {
            'focusable': 'false',
            'preserveAspectRatio': 'xMidYMid meet',
            'fill': 'currentColor',
            'style': {
                'width': '%spx' % 20,
                'height': '%spx' % 20,
            },
            'aria-hidden': 'true',
            'class': 'bx--navigation-menu-panel-collapse-icon',
        })


class NavSection(Node):
    """Side navigation's section.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--navigation-section {class}" {props}>
  <ul class="bx--navigation-items">
    {child}
  </ul>
</{tag}>
"""
        return self.format(template, values)


class NavItem(Node):
    """Side navigation's items.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('submenu', 'icon')
    "Named children."
    NODE_PROPS = ('id', 'active', 'icon_size')
    "Extended Template Tag arguments."

    is_submenu = False

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        is_active = self.eval(self.kwargs.get('active'), context)

        self.is_submenu = context.get('navitem_submenu')
        if self.is_submenu:
            values['class'].append('bx--navigation__category-item')
            if is_active:
                values['class'].append('bx--navigation__category-item--active')
        else:
            values['class'].append('bx--navigation-item')
            if is_active:
                values['class'].append('bx--navigation-item--active')

        if 'submenu' in self.slots:
            context['navitem_submenu'] = True

        if 'icon' in self.slots:
            values['class'].append('bx--navigation-item--icon')


    def render_default(self, values, context):
        """Output html of the component.
        """
        if 'submenu' in self.slots:
            template = """
<li class="{class}" {props}>
  <div class="bx--navigation__category">
    <button type="button" class="bx--navigation__category-toggle"
        aria-haspopup="true" aria-expanded="false"
        aria-controls="category-{id}-menu">
      {slot_icon}
      <div class="bx--navigation__category-title">
        {child}
        <svg aria-hidden="true" width="20" height="20"
            xmlns="http://www.w3.org/2000/svg" viewBox="0 0 32 32">
          <path d="M16 22L6 12l1.414-1.414L16 19.172l8.586-8.586L26 12 16 22z" />
        </svg>
      </div>
    </button>
    {slot_submenu}
  </div>
</li>
"""
        else:
            template = """
<li class="{class}">
  <a class="bx--navigation-link" {props}>
    {slot_icon}
    {child}
  </a>
</li>
"""
        return self.format(template, values, context)


    def render_slot_submenu(self, values, context):
        """Render html of the slot.
        """
        template = """
<ul id="category-{id}-menu" class="bx--navigation__category-items {class}"
    {props}>
  {child}
</ul>
"""
        return self.format(template, values)


    def render_slot_icon(self, values, context):
        """Render html of the slot.
        """
        size = self.eval(self.kwargs.get('icon_size', 20), context)
        values['child'] = modify_svg(values['child'], {
            'focusable': 'false',
            'preserveAspectRatio': 'xMidYMid meet',
            'fill': 'currentColor',
            'style': {
                'width': '%spx' % size,
                'height': '%spx' % size,
            },
            'aria-hidden': 'true',
        })

        template = """
<div class="bx--navigation-icon {class}" {props}>
  {child}
</div>
"""
        return self.format(template, values)


class SideNav(Node):
    """UI Shell side navigation.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('title_icon', 'title', 'switcher')
    "Named children."
    NODE_PROPS = ('id', 'fixed',)
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'aside'
    "Rendered HTML tag."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_label'] = _("Side navigation")
        values['txt_close'] = _("Close the side navigation menu")
        values['txt_close_help'] = _("Toggle the expansion state of the navigation") # pylint:disable=line-too-long

        if self.eval(self.kwargs.get('fixed'), context):
            values['class'].append('bx--side-nav--fixed')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<{tag} class="bx--side-nav {class}" data-side-nav {props}>
  <nav class="bx--side-nav__navigation" role="navigation"
      aria-label="{txt_label}">
    {tmpl_header}
    <ul class="bx--side-nav__items">
      {child}
    </ul>
    <footer class="bx--side-nav__footer {class}" {props}>
      <button type="button" class="bx--side-nav__toggle" title="{txt_close}">
        <div class="bx--side-nav__icon">
          <svg focusable="false" preserveAspectRatio="xMidYMid meet"
              xmlns="http://www.w3.org/2000/svg" fill="currentColor"
              class="bx--side-nav__icon--collapse bx--side-nav-collapse-icon"
              width="20" height="20" viewBox="0 0 32 32" aria-hidden="true">
            <path d="M24 9.4L22.6 8 16 14.6 9.4 8 8 9.4 14.6 16 8 22.6 9.4 24 16 17.4 22.6 24 24 22.6 17.4 16 24 9.4z"></path>
          </svg>
          <svg focusable="false" preserveAspectRatio="xMidYMid meet"
              xmlns="http://www.w3.org/2000/svg" fill="currentColor"
              class="bx--side-nav__icon--expand bx--side-nav-expand-icon"
              width="20" height="20" viewBox="0 0 32 32" aria-hidden="true">
            <path d="M22 16L12 26 10.6 24.6 19.2 16 10.6 7.4 12 6z"></path>
          </svg>
        </div>
        <span class="bx--assistive-text">{txt_close_help}</span>
      </button>
    </footer>
  </nav>
</{tag}>
"""
        return self.format(template, values, context)


    def render_slot_title_icon(self, values, context):
        """Render html of the slot.
        """
        values['child'] = modify_svg(values['child'], {
            'preserveAspectRatio': 'xMidYMid meet',
            'fill': 'currentColor',
            'style': {
                'width': '%spx' % 20,
                'height': '%spx' % 20,
            },
            'aria-hidden': 'true',
        })
        template = """
<div class="bx--side-nav__icon {class}">
  {child}
</div>
"""
        return self.format(template, values)


    def render_slot_title(self, values, context):
        """Render html of the slot.
        """
        values['cleaned_title'] = clean_attr_value(values['child'])

        template = """
<h2 class="bx--side-nav__title {class}" title="{cleaned_title}" {props}>
  {child}
</h2>
"""
        return self.format(template, values)


    def render_slot_switcher(self, values, context):
        """Render html of the slot.
        """
        values['txt_switcher'] = _("Switcher")

        template = """
<div class="bx--side-nav__switcher {class}">
  <label for="side-nav-switcher-{id}" class="bx--assistive-text">
    {txt_switcher}
  </label>
  <select id="side-nav-switcher-{id}" class="bx--side-nav__select" {props}>
    <option class="bx--side-nav__option" value="">{txt_switcher}</option>
    {child}
  </select>
  <div class="bx--side-nav__switcher-chevron">
    <svg aria-hidden="true" width="20" height="20"
        xmlns="http://www.w3.org/2000/svg" viewBox="0 0 32 32">
      <path d="M16 22L6 12l1.414-1.414L16 19.172l8.586-8.586L26 12 16 22z" />
    </svg>
  </div>
</div>
"""
        return self.format(template, values)


    def render_tmpl_header(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if set(('title_icon', 'title', 'switcher'))\
                .intersection(self.slots.keys()):

            template = """
<header class="bx--side-nav__header {class}" {props}>
  {slot_title_icon}
  <div class="bx--side-nav__details">
    {slot_title}
    {slot_switcher}
  </div>
</header>
"""
            return self.format(template, values, context)
        return ""


class SideNavOption(Node):
    """UI Shell side navigation's switcher option.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."

    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<option class="bx--side-nav__option {class}" {props}>
  {child}
</option>
"""
        return self.format(template, values)


class SideNavItem(Node):
    """UI Shell side navigation's items.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('submenu', 'icon')
    "Named children."
    NODE_PROPS = ('active', 'current', 'icon_size')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('link',)
    "Prepare xxx_class and xxx_props values."

    is_submenu = False

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        is_active = self.eval(self.kwargs.get('active'), context)

        self.is_submenu = context.get('navitem_submenu')
        if self.is_submenu:
            values['class'].append('bx--side-nav__menu-item')
            if is_active:
                values['class'].append('bx--side-nav__menu-item--active')
        else:
            values['class'].append('bx--side-nav__item')
            if is_active:
                values['class'].append('bx--side-nav__item--active')

        if self.eval(self.kwargs.get('current'), context):
            values['link_class'].append('bx--side-nav__link--current')
            values['props'].append(('aria-current', 'page'))

        if 'submenu' in self.slots:
            context['navitem_submenu'] = True
        else:
            values['icon_class'].append('bx--side-nav__icon--small')


    def render_default(self, values, context):
        """Output html of the component.
        """
        if 'submenu' in self.slots:
            template = """
<li class="{class}">
  <button type="button" class="bx--side-nav__submenu" aria-haspopup="true"
      aria-expanded="true" aria-controls="sidenav-{id}-menu">
    {slot_icon}
    <span class="bx--side-nav__submenu-title">
      {child}
    </span>
    <div class="bx--side-nav__icon bx--side-nav__icon--small bx--side-nav__submenu-chevron">
      <svg aria-hidden="true" width="20" height="20"
          xmlns="http://www.w3.org/2000/svg" viewBox="0 0 32 32">
        <path d="M16 22L6 12l1.414-1.414L16 19.172l8.586-8.586L26 12 16 22z" />
      </svg>
    </div>
  </button>
  <ul id="sidenav-{id}-menu" class="bx--side-nav__menu" {props}>
    {slot_submenu}
  </ul>
</li>
"""
        elif self.is_submenu:
            template = """
<li role="none" class="{class}">
  <a class="bx--side-nav__link {link_class}" {props}>
    {slot_icon}
    <span class="bx--side-nav__link-text">{child}</span>
  </a>
</li>
"""
        else:
            template = """
<li class="{class}">
  <a class="bx--side-nav__link {link_class}" {props}>
    {slot_icon}
    <span class="bx--side-nav__link-text">{child}</span>
  </a>
</li>
"""
        return self.format(template, values, context)


    def render_slot_icon(self, values, context):
        """Render html of the slot.
        """
        size = self.eval(self.kwargs.get('icon_size', 20), context)
        values['child'] = modify_svg(values['child'], {
            'preserveAspectRatio': 'xMidYMid meet',
            'fill': 'currentColor',
            'style': {
                'width': '%spx' % size,
                'height': '%spx' % size,
            },
            'aria-hidden': 'true',
        })
        template = """
<div class="bx--side-nav__icon {class}" {props}>
  {child}
</div>
"""
        return self.format(template, values)


components = {
    'UiShell': UiShell,
    'UiLink': Link,
    'UiAction': Action,
    'UiNavSection': NavSection,
    'UiNavItem': NavItem,
    'UiSideNav': SideNav,
    'UiSideNavOption': SideNavOption,
    'UiSideNavItem': SideNavItem,
}



"""
Text Input
==========

See: https://www.carbondesignsystem.com/components/text-input/usage/

Text inputs enable the user to interact with and input content and data. This
component can be used for long and short form entries.

Overview
--------

There are instances, sometimes in the same form, where you need users to enter
both short and long form content. Allow the size of the text input box to
reflect the length of the content you expect the user to enter.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.forms.widgets import PasswordInput as PasswordWidget
from django.utils.translation import gettext as _
#-
from .base import FormNode, DumbFormNode

class TextInput(FormNode):
    """Text Input component.
    """
    NODE_PROPS = ('light',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('disabled'), context):
            values['label_class'].append('bx--label--disabled')
            values['help_class'].append('bx--form__helper-text--disabled')


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].append('bx--text-input')

        if self.eval(self.kwargs.get('light'), context):
            props['class'].append('bx--text-input--light')

        if self.bound_field.errors:
            props['class'].append('bx--text-input--invalid')


    def render_default(self, values, context):
        """Output html of the component.
        """
        if self.bound_field.errors:
            template = """
<div class="bx--form-item bx--text-input-wrapper">
  {tmpl_label}
  <div class="bx--text-input__field-wrapper" data-invalid>
    {tmpl_icon_invalid}
    {tmpl_element}
  </div>
  <div class="bx--form-requirement">
    {tmpl_errors}
  </div>
  {tmpl_help}
</div>
"""
        else:
            template = """
<div class="bx--form-item bx--text-input-wrapper">
  {tmpl_label}
  <div class="bx--text-input__field-wrapper">
    {tmpl_element}
  </div>
  {tmpl_help}
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_icon_invalid(self, values, context):
        """Dynamically render a part of the component's template.
        """
        return """
<svg focusable="false" preserveAspectRatio="xMidYMid meet"
    xmlns="http://www.w3.org/2000/svg" fill="currentColor"
    class="bx--text-input__invalid-icon" width="16" height="16"
    viewBox="0 0 16 16" aria-hidden="true">
  <path d="M8,1C4.2,1,1,4.2,1,8s3.2,7,7,7s7-3.1,7-7S11.9,1,8,1z M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2	c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z"></path>
  <path d="M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8	c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z" data-icon-path="inner-path" opacity="0"></path>
</svg>
"""


class PasswordInput(TextInput):
    """Password Input component.
    """
    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        super().prepare(values, context)
        values['txt_show_password'] = _("Show password")

        widget = self.bound_field.field.widget
        if not isinstance(widget, PasswordWidget):
            self.bound_field.field.widget = PasswordWidget(widget.attrs)


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        super().prepare_element_props(props, context)
        props['class'].append('bx--password-input')
        props['data-toggle-password-visibility'] = ''


    def render_default(self, values, context):
        """Output html of the component.
        """
        if self.bound_field.errors:
            template = """
<div data-text-input
    class="bx--form-item bx--text-input-wrapper bx--password-input-wrapper">
  {tmpl_label}
  <div class="bx--text-input__field-wrapper" data-invalid>
    {tmpl_icon_invalid}
    {tmpl_element}
    {tmpl_visibility}
  </div>
  <div class="bx--form-requirement">
    {tmpl_errors}
  </div>
  {tmpl_help}
</div>
"""
        else:
            template = """
<div data-text-input
    class="bx--form-item bx--text-input-wrapper bx--password-input-wrapper">
  {tmpl_label}
  <div class="bx--text-input__field-wrapper">
    {tmpl_element}
    {tmpl_visibility}
  </div>
  {tmpl_help}
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_visibility(self, values, context):
        """Dynamically render a part of the component's template.
        """
        template = """
<button type="button"
    class="bx--text-input--password__visibility__toggle bx--tooltip__trigger bx--tooltip--a11y bx--tooltip--bottom bx--tooltip--align-center">
  <span class="bx--assistive-text">{txt_show_password}</span>
  <svg focusable="false" preserveAspectRatio="xMidYMid meet"
      xmlns="http://www.w3.org/2000/svg" fill="currentColor"
      hidden="true" class="bx--icon--visibility-off" width="16" height="16"
      viewBox="0 0 16 16" aria-hidden="true">
    <path d="M2.6,11.3l0.7-0.7C2.6,9.8,1.9,9,1.5,8c1-2.5,3.8-4.5,6.5-4.5c0.7,0,1.4,0.1,2,0.4l0.8-0.8C9.9,2.7,9,2.5,8,2.5	C4.7,2.6,1.7,4.7,0.5,7.8c0,0.1,0,0.2,0,0.3C1,9.3,1.7,10.4,2.6,11.3z"></path>
    <path d="M6 7.9c.1-1 .9-1.8 1.8-1.8l.9-.9C7.2 4.7 5.5 5.6 5.1 7.2 5 7.7 5 8.3 5.1 8.8L6 7.9zM15.5 7.8c-.6-1.5-1.6-2.8-2.9-3.7L15 1.7 14.3 1 1 14.3 1.7 15l2.6-2.6c1.1.7 2.4 1 3.7 1.1 3.3-.1 6.3-2.2 7.5-5.3C15.5 8.1 15.5 7.9 15.5 7.8zM10 8c0 1.1-.9 2-2 2-.3 0-.7-.1-1-.3L9.7 7C9.9 7.3 10 7.6 10 8zM8 12.5c-1 0-2.1-.3-3-.8l1.3-1.3c1.4.9 3.2.6 4.2-.8.7-1 .7-2.4 0-3.4l1.4-1.4c1.1.8 2 1.9 2.6 3.2C13.4 10.5 10.6 12.5 8 12.5z"></path>
  </svg>
  <svg focusable="false" preserveAspectRatio="xMidYMid meet"
      xmlns="http://www.w3.org/2000/svg" fill="currentColor"
      class="bx--icon--visibility-on" width="16" height="16"
      viewBox="0 0 16 16" aria-hidden="true">
    <path d="M15.5,7.8C14.3,4.7,11.3,2.6,8,2.5C4.7,2.6,1.7,4.7,0.5,7.8c0,0.1,0,0.2,0,0.3c1.2,3.1,4.1,5.2,7.5,5.3	c3.3-0.1,6.3-2.2,7.5-5.3C15.5,8.1,15.5,7.9,15.5,7.8z M8,12.5c-2.7,0-5.4-2-6.5-4.5c1-2.5,3.8-4.5,6.5-4.5s5.4,2,6.5,4.5	C13.4,10.5,10.6,12.5,8,12.5z"></path>
    <path d="M8,5C6.3,5,5,6.3,5,8s1.3,3,3,3s3-1.3,3-3S9.7,5,8,5z M8,10c-1.1,0-2-0.9-2-2s0.9-2,2-2s2,0.9,2,2S9.1,10,8,10z"></path>
  </svg>
</button>
"""
        return self.format(template, values)


class DumbTextInput(DumbFormNode):
    """Text Input component but without Django form.
    """
    NODE_PROPS = ('light',)
    "Extended Template Tag arguments."


components = {
    'PasswordInput': PasswordInput,
    'TextInput': TextInput,
    'DumbTextInput': DumbTextInput,
}



"""
Progress Indicator
==================

See: https://www.carbondesignsystem.com/components/progress-indicator/usage/

A progress indicator is a visual representation of a user’s progress through
a set of steps, guiding toward the completion of a specified process.

Overview
--------

Use progress indicators to keep the user on track when completing a specific
task. By dividing the end goal into smaller, sub-tasks, it increases the
percentage of completeness as each task is completed.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from .base import Node

class ProgressIndicator(Node):
    """Progress Indicator component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('vertical',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('vertical'), context):
            values['class'].append('bx--progress--vertical')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<ul data-progress data-progress-current class="bx--progress {class}" {props}>
  {child}
</ul>
"""
        return self.format(template, values, context)


class ProgressIndicatorItem(Node):
    """Progress Indicator item component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('help', 'optional')
    "Named children."
    NODE_PROPS = ('id', 'variant',)
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('list',)
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_VARIANT = ('current', 'complete', 'incomplete', 'invalid',
            'disabled')
    "Documentation only."

    variant = None

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        self.variant = variant = self.eval(
                self.kwargs.get('variant', 'current'), context)

        if variant == 'invalid':
            values['list_class'].append('bx--progress-step--incomplete')
            values['list_props'].append(('data-invalid', True))
        else:
            values['list_class'].append(f'bx--progress-step--{variant}')

        if variant == 'disabled':
            values['list_class'].append('bx--progress-step--incomplete')
            values['list_props'].append(('aria-disabled', 'true'))

        if 'help' in self.slots:
            values['props'].append(('aria-describedby', f'hint-{self._id}'))


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<li class="bx--progress-step {list_class}" {list_props}>
  {tmpl_icon}
  <p tabindex="0" class="bx--progress-label {class}" id="{id}" {props}>
    {child}
  </p>
  {slot_help}
  {slot_optional}
  <span class="bx--progress-line"></span>
</li>
"""
        return self.format(template, values, context)


    def render_slot_help(self, values, context):
        """Render html of the slot.
        """
        template = """
<div id="hint-{id}" role="tooltip" data-floating-menu-direction="bottom"
    class="bx--tooltip {class}" data-avoid-focus-on-open {props}>
  <span class="bx--tooltip__caret"></span>
  <p class="bx--tooltip__text">{child}</p>
</div>
"""
        return self.format(template, values)


    def render_slot_optional(self, values, context):
        """Render html of the slot.
        """
        template = """
<p class="bx--progress-optional {class}" {props}>{child}</p>
"""
        return self.format(template, values)


    def render_tmpl_icon(self, values, context):
        """Dynamically render a part of the component's template.
        """
        if self.variant == 'current':
            return """
<svg>
  <path d="M 7, 7 m -7, 0 a 7,7 0 1,0 14,0 a 7,7 0 1,0 -14,0"></path>
</svg>
"""
        if self.variant == 'complete':
            return """
<svg focusable="false" preserveAspectRatio="xMidYMid meet"
    xmlns="http://www.w3.org/2000/svg" fill="currentColor" width="16"
    height="16" viewBox="0 0 32 32" aria-hidden="true">
  <path d="M14 21.414L9 16.413 10.413 15 14 18.586 21.585 11 23 12.415 14 21.414z"></path>
  <path d="M16,2A14,14,0,1,0,30,16,14,14,0,0,0,16,2Zm0,26A12,12,0,1,1,28,16,12,12,0,0,1,16,28Z"></path>
</svg>
"""
        if self.variant == 'incomplete':
            return """
<svg>
  <path d="M8 1C4.1 1 1 4.1 1 8s3.1 7 7 7 7-3.1 7-7-3.1-7-7-7zm0 13c-3.3 0-6-2.7-6-6s2.7-6 6-6 6 2.7 6 6-2.7 6-6 6z"></path>
</svg>
"""
        if self.variant == 'invalid':
            return """
<svg focusable="false" preserveAspectRatio="xMidYMid meet"
    xmlns="http://www.w3.org/2000/svg" fill="currentColor"
    class="bx--progress__warning" width="16" height="16" viewBox="0 0 16 16"
    aria-hidden="true">
  <path d="M8,1C4.1,1,1,4.1,1,8s3.1,7,7,7s7-3.1,7-7S11.9,1,8,1z M8,14c-3.3,0-6-2.7-6-6s2.7-6,6-6s6,2.7,6,6S11.3,14,8,14z"></path>
  <path d="M7.5 4H8.5V9H7.5zM8 10.2c-.4 0-.8.3-.8.8s.3.8.8.8c.4 0 .8-.3.8-.8S8.4 10.2 8 10.2z"></path>
</svg>
"""
        if self.variant == 'disabled':
            return """
<svg>
  <path d="M8 1C4.1 1 1 4.1 1 8s3.1 7 7 7 7-3.1 7-7-3.1-7-7-7zm0 13c-3.3 0-6-2.7-6-6s2.7-6 6-6 6 2.7 6 6-2.7 6-6 6z"></path>
</svg>
"""
        return ''


components = {
    'ProgressIndicator': ProgressIndicator,
    'Progress': ProgressIndicatorItem,
}



"""
List Box
========

See: https://the-carbon-components.netlify.app/?nav=list-box

""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import Node, FormNode

class ListBox(FormNode):
    """List Box component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    MODES = ('default', 'inline')
    "Available variants."
    NODE_PROPS = ('light',)
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('label', 'help', 'list')
    "Prepare xxx_class and xxx_props values."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_open'] = _("Open menu")

        if self.eval(self.kwargs.get('disabled'), context):
            values['label_class'].append('bx--label--disabled')
            values['help_class'].append('bx--form__helper-text--disabled')
            values['list_class'].append('bx--list-box--disabled')

        if self.eval(self.kwargs.get('light'), context):
            values['list_class'].append('bx--list-box--light')


    def prepare_element_props(self, props, context):
        """Prepare html attributes for rendering the form element.
        """
        props['class'].append('bx--text-input')
        props['aria-autocomplete'] = 'list'
        props['aria-expanded'] = 'false'
        props['autocomplete'] = 'off'
        props['aria-owns'] = 'menu-' + self._id


    def render_default(self, values, context):
        """Output html of the component.
        """
        if self.bound_field.errors:
            template = """
<div class="bx--form-item">
  <div class="bx--list-box__wrapper">
    {tmpl_label}
    <div class="bx--list-box {list_class}" data-invalid>
      {tmpl_icon_invalid}
      <div role="button" class="bx--list-box__field" tabindex="0"
          aria-label="{txt_open}" aria-expanded="false" aria-haspopup="true">
        <span class="bx--list-box__label">{value}</span>
        {tmpl_icon_menu}
      </div>
      <ul class="bx--list-box__menu" role="combobox" id="menu-{id}">
        {child}
      </ul>
    </div>
    {tmpl_help}
    <div class="bx--form-requirement">
      {form_errors}
    </div>
  </div>
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  <div class="bx--list-box__wrapper">
    {tmpl_label}
    <div class="bx--list-box {list_class}">
      <div role="button" class="bx--list-box__field" tabindex="0"
          aria-label="{txt_open}" aria-expanded="false" aria-haspopup="true">
        <span class="bx--list-box__label">{value}</span>
        {tmpl_icon_menu}
      </div>
      <ul class="bx--list-box__menu" role="combobox" id="menu-{id}">
        {child}
      </ul>
    </div>
    {tmpl_help}
  </div>
</div>
"""
        return self.format(template, values, context)


    def render_inline(self, values, context):
        """Output html of the component.
        """
        if self.bound_field.errors:
            template = """
<div class="bx--form-item">
  <div class="bx--list-box__wrapper bx--list-box__wrapper--inline">
    {tmpl_label}
    <div class="bx--list-box bx--list-box--inline {list_class}" data-invalid>
      {tmpl_icon_invalid}
      <div role="button" class="bx--list-box__field" tabindex="0"
          aria-label="{txt_open}" aria-expanded="false" aria-haspopup="true">
        <span class="bx--list-box__label">{value}</span>
        {tmpl_icon_menu}
      </div>
      <ul class="bx--list-box__menu" role="combobox" id="menu-{id}">
        {child}
      </ul>
    </div>
    {tmpl_help}
    <div class="bx--form-requirement">
      {form_errors}
    </div>
  </div>
</div>
"""
        else:
            template = """
<div class="bx--form-item">
  <div class="bx--list-box__wrapper bx--list-box__wrapper--inline">
    {tmpl_label}
    <div class="bx--list-box bx--list-box--inline {list_class}">
      <div role="button" class="bx--list-box__field" tabindex="0"
          aria-label="{txt_open}" aria-expanded="false" aria-haspopup="true">
        <span class="bx--list-box__label">{value}</span>
        {tmpl_icon_menu}
      </div>
      <ul class="bx--list-box__menu" role="combobox" id="menu-{id}">
        {child}
      </ul>
    </div>
    {tmpl_help}
  </div>
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_label(self, values, context):
        """Dynamically render a part of the component's template.
        """
        template = """
<label for="{id}" class="bx--label {label_class}" {label_props}>
  {label}
</label>
"""
        return self.format(template, values)


    def render_tmpl_icon_invalid(self, values, context):
        """Dynamically render a part of the component's template.
        """
        return """
<svg focusable="false" preserveAspectRatio="xMidYMid meet"
    xmlns="http://www.w3.org/2000/svg" fill="currentColor"
    class="bx--list-box__invalid-icon" width="16" height="16"
    viewBox="0 0 16 16" aria-hidden="true">
  <path d="M8,1C4.2,1,1,4.2,1,8s3.2,7,7,7s7-3.1,7-7S11.9,1,8,1z M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2   c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z"></path>
  <path d="M7.5,4h1v5h-1C7.5,9,7.5,4,7.5,4z M8,12.2c-0.4,0-0.8-0.4-0.8-0.8s0.3-0.8,0.8-0.8 c0.4,0,0.8,0.4,0.8,0.8S8.4,12.2,8,12.2z" data-icon-path="inner-path" opacity="0"></path>
</svg>
"""


    def render_tmpl_icon_menu(self, values, context):
        """Dynamically render a part of the component's template.
        """
        return """
<div class="bx--list-box__menu-icon">
  <svg focusable="false" preserveAspectRatio="xMidYMid meet"
      xmlns="http://www.w3.org/2000/svg" fill="currentColor"
      aria-label="{txt_open}" width="16" height="16" viewBox="0 0 16 16"
      role="img">
    <path d="M8 11L3 6 3.7 5.3 8 9.6 12.3 5.3 13 6z"></path>
  </svg>
</div>
"""


class ListBoxItem(Node):
    """List Box item component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('active',)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('active'), context):
            values['class'].append('bx--list-box__menu-item--active')
            values['class'].append('bx--list-box__menu-item--highlighted')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<li class="bx--list-box__menu-item {class}" {props}>
  <div class="bx--list-box__menu-item__option" tabindex="0">
    {child}
    <svg focusable="false" preserveAspectRatio="xMidYMid meet"
        xmlns="http://www.w3.org/2000/svg" fill="currentColor"
        aria-hidden="true" class="bx--list-box__menu-item__selected-icon"
        width="16" height="16" viewBox="0 0 32 32">
      <path d="M13 24L4 15 5.414 13.586 13 21.171 26.586 7.586 28 9 13 24z"></path>
    </svg>
  </div>
</li>
"""
        return self.format(template, values)


components = {
    'ListBox': ListBox,
    'ListBoxItem': ListBoxItem,
}



"""
Modal
=====

See: https://www.carbondesignsystem.com/components/modal/usage/

Modals focus the user’s attention exclusively on one task or piece of
information via a window that sits on top of the page content.

Overview
--------

Modals are a variant of dialog used to present critical information or request
user input needed to complete a user’s workflow. Modals interrupt a user’s
workflow by design. When active, a user is blocked from the on-page content
and cannot return to their previous workflow until the modal task is completed
or the user dismisses the modal. While effective when used correctly, modals
should be used sparingly to limit disruption to the user.

Modal dialogs are commonly used for short and non-frequent tasks, such as
editing or management tasks. If a user needs to repeatably preform a task,
consider making the task do-able from the main page.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import Node
from .button import Button

class Modal(Node):
    """Modal component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    SLOTS = ('label', 'heading', 'footer')
    "Named children."
    NODE_PROPS = ('id', 'variant', 'has_form', 'size', 'can_scroll')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('container', 'content', 'close')
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_VARIANT = ('danger',)
    "Documentation only."
    POSSIBLE_SIZE = ('xs', 'sm', 'lg')
    "Documentation only."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_close'] = _("close modal")

        variant = self.eval(self.kwargs.get('variant'), context)
        if variant:
            values['class'].append(f'bx--modal--{variant}')

        id_ = values['id']
        if 'label' in self.slots:
            values['props'].append(('aria-labelledby', f'label-{id_}'))
        if 'heading' in self.slots:
            values['props'].append(('aria-describedby', f'heading-{id_}'))

        has_form = self.eval(self.kwargs.get('has_form'), context)
        if has_form:
            values['content_class'].append('bx--modal-content--with-form')

        size = self.eval(self.kwargs.get('size'), context)
        if size:
            values['container_class'].append(f'bx--modal-container--{size}')

        if not has_form and 'footer' not in self.slots:
            values['close_props'].append(('data-modal-primary-focus', True))

        if self.eval(self.kwargs.get('can_scroll'), context):
            values['content_props'].append(('tabindex', '0'))


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div data-modal id="{id}" class="bx--modal {class}" role="dialog"
    aria-modal="true" tabindex="-1" {props}>
  <div class="bx--modal-container {container_class}" {container_props}>
    <div class="bx--modal-header">
      {slot_label}
      {slot_heading}
      <button class="bx--modal-close {close_class}" type="button" data-modal-close
          aria-label="{txt_close}" {close_props}>
        <svg focusable="false" preserveAspectRatio="xMidYMid meet"
            xmlns="http://www.w3.org/2000/svg" fill="currentColor"
            class="bx--modal-close__icon" width="16" height="16"
            viewBox="0 0 32 32" aria-hidden="true">
          <path d="M24 9.4L22.6 8 16 14.6 9.4 8 8 9.4 14.6 16 8 22.6 9.4 24 16 17.4 22.6 24 24 22.6 17.4 16 24 9.4z"></path>
        </svg>
      </button>
    </div>

    <div class="bx--modal-content {content_class}" {content_props}>
      {child}
    </div>
    <div class="bx--modal-content--overflow-indicator"></div>

    {slot_footer}
  </div>
  <span tabindex="0"></span>
</div>
"""
        return self.format(template, values, context)


    def render_slot_label(self, values, context):
        """Render html of the slot.
        """
        template = """
<p class="bx--modal-header__label bx--type-delta {class}" id="label-{id}"
    {props}>
  {child}
</p>
"""
        return self.format(template, values)


    def render_slot_heading(self, values, context):
        """Render html of the slot.
        """
        template = """
<p class="bx--modal-header__heading bx--type-beta {class}" id="heading-{id}"
    {props}>
  {child}
</p>
"""
        return self.format(template, values)


    def render_slot_footer(self, values, context):
        """Render html of the slot.
        """
        template = '<div class="bx--modal-footer {class}" {props}>{child}</div>'
        return self.format(template, values)


class ModalTrigger(Button):
    """Modal trigger button.
    """
    NODE_PROPS = ('target', *Button.NODE_PROPS)
    "Extended Template Tag arguments."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        super().prepare(values, context)

        target = self.eval(self.kwargs['target'], context)
        values['props'].append(('data-modal-target', f'#{target}'))


    def after_prepare(self, values, context):
        """Simplifying values meant for rendering templates.
        """
        super().after_prepare(values, context)

        if not values['child']:
            values['child'] = _("Show modal")


components = {
    'Modal': Modal,
    'ModalTrigger': ModalTrigger,
}



"""
Overflow Menu
=============

See: https://www.carbondesignsystem.com/components/overflow-menu/usage/

Use the overflow menu component when additional options are available to the
user but there is a space constraint.

Overview
--------

Overflow menu is used when additional options are available to the user and
there is a space constraint.
""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import Node, clean_attr_value

class OverflowMenu(Node):
    """UI Shell component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('id', 'flip', 'direction', 'position', 'align')
    "Extended Template Tag arguments."
    CLASS_AND_PROPS = ('trigger',)
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_DIRECTION = ('top', 'bottom')
    "Documentation only."
    POSSIBLE_POSITION = ('left', 'right')
    "Documentation only."
    POSSIBLE_ALIGN = ('start',)
    "Documentation only."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_overflow'] = _("Overflow")

        if self.eval(self.kwargs.get('flip'), context):
            values['class'].append('bx--overflow-menu--flip')

        direction = self.eval(self.kwargs.get('direction', 'bottom'), context)
        values['props'].append(('data-floating-menu-direction', direction))

        position = self.eval(self.kwargs.get('position', 'right'), context)
        values['trigger_class'].append(f'bx--tooltip--{position}')

        align = self.eval(self.kwargs.get('align', 'start'), context)
        values['trigger_class'].append(f'bx--tooltip--align-{align}')


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<div data-overflow-menu class="bx--overflow-menu">
  <button class="bx--overflow-menu__trigger bx--tooltip__trigger bx--tooltip--a11y {trigger_class}"
      aria-haspopup="true" aria-expanded="false" id="trigger-{id}"
      aria-controls="{id}" {trigger_props}>
    <span class="bx--assistive-text">{txt_overflow}</span>
    {tmpl_icon}
  </button>
  <div class="bx--overflow-menu-options {class}" tabindex="-1" role="menu"
      aria-labelledby="trigger-{id}" id="{id}" {props}>
    <ul class="bx--overflow-menu-options__content">
      {child}
    </ul>
    <span tabindex="0"></span>
  </div>
</div>
"""
        return self.format(template, values, context)


    def render_tmpl_icon(self, values, context):
        """Dynamically render a part of the component's template.
        """
        return """
<svg focusable="false" preserveAspectRatio="xMidYMid meet"
    xmlns="http://www.w3.org/2000/svg" fill="currentColor"
    class="bx--overflow-menu__icon" width="16" height="16" viewBox="0 0 32 32"
    aria-hidden="true">
  <circle cx="16" cy="8" r="2"></circle>
  <circle cx="16" cy="16" r="2"></circle>
  <circle cx="16" cy="24" r="2"></circle>
</svg>
"""


class OverflowMenuItem(Node):
    """Overflow Menu item component.
    """
    WANT_CHILDREN = True
    "Template Tag needs closing end tag."
    NODE_PROPS = ('disabled', 'active', 'variant')
    "Extended Template Tag arguments."
    DEFAULT_TAG = 'button'
    "Rendered HTML tag."
    CLASS_AND_PROPS = ('list',)
    "Prepare xxx_class and xxx_props values."
    POSSIBLE_VARIANT = ('danger',)
    "Documentation only."

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        if self.eval(self.kwargs.get('disabled'), context):
            values['list_class'].append(
                    'bx--overflow-menu-options__option--disabled')
            if values['tag'] == 'button':
                values['props'].append(('disabled', True))
            else:
                values['props'].append(('tabindex', '-1'))
                values['props'].append(('aria-disabled', 'true'))

        if self.eval(self.kwargs.get('active'), context):
            values['props'].append(('data-floating-menu-primary-focus', True))

        variant = self.eval(self.kwargs.get('variant'), context)
        if variant:
            values['list_class'].append(
                    f'bx--overflow-menu-options__option--{variant}')


    def after_prepare(self, values, context):
        """Simplifying values meant for rendering templates.
        """
        if self.eval(self.kwargs.get('active'), context) and\
                'title' not in self.kwargs:
            values['props'].append(('title', clean_attr_value(values['child'])))
        super().after_prepare(values, context)


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<li class="bx--overflow-menu-options__option {list_class}" {list_props}>
  <{tag} class="bx--overflow-menu-options__btn {class}" role="menuitem" {props}>
    <span class="bx--overflow-menu-options__option-content">
      {child}
    </span>
  </{tag}>
</li>
"""
        return self.format(template, values)


components = {
    'OvMenu': OverflowMenu,
    'OvMenuItem': OverflowMenuItem,
}



"""
Copy Button
===========

See: https://the-carbon-components.netlify.app/?nav=copy-button

""" # pylint:disable=line-too-long
# pylint:disable=too-many-lines

from django.utils.translation import gettext as _
#-
from .base import Node

class CopyButton(Node):
    """Copy Button component.
    """

    def prepare(self, values, context):
        """Prepare values for rendering the templates.
        """
        values['txt_copied'] = _("Copied!")


    def render_default(self, values, context):
        """Output html of the component.
        """
        template = """
<button data-copy-btn class="bx--copy-btn {class}" type="button" tabindex="0"
    {props}>
  <span class="bx--assistive-text bx--copy-btn__feedback">{txt_copied}</span>
  <svg focusable="false" preserveAspectRatio="xMidYMid meet"
      xmlns="http://www.w3.org/2000/svg" fill="currentColor"
      class="bx--snippet__icon" width="16" height="16" viewBox="0 0 32 32"
      aria-hidden="true">
    <path d="M28,10V28H10V10H28m0-2H10a2,2,0,0,0-2,2V28a2,2,0,0,0,2,2H28a2,2,0,0,0,2-2V10a2,2,0,0,0-2-2Z"></path>
    <path d="M4,18H2V4A2,2,0,0,1,4,2H18V4H4Z"></path>
  </svg>
</button>
"""
        return self.format(template, values)


components = {
    'CopyButton': CopyButton,
}
